@@ -4,11 +4,32 @@
 /* Author: Rainer Doemer			first version: 07/02/97 */
 /************************************************************************/
 
-/* last update: 08/06/01 */
+/* last update: 09/26/06 */
 
 /* modifications: (most recent first)
  *
- * 08/06/01 RD	adjusted version to 2.1.1
+ * 09/26/06 PC  Adjustments for scrc 2.1
+ * 06/03/05 RD	reorganized and renamed global type names
+ * 01/01/05 RD	extended copyright to 2005
+ * 12/16/04 RD	added GL_ERROR_FILE_NOT_WRITABLE
+ * 06/23/04 RD	increased version to 2.2.0
+ * 06/15/04 PC  Adjustments for scrc 2.0
+ * 12/17/03 RD	increased version to 2.2.c, extended copyright to 2004
+ * 12/16/02 RD	increased version to 2.2.b, extended copyright to 2003
+ * 10/23/02 RD	reallocated error ranges, added entry for OS tool
+ * 09/16/02 RD	downgraded the version number to 2.2.a for an alpha release
+ * 07/19/02 RD	increased version to 2.2.0
+ * 07/05/02 RD	increased SIR_ERROR_RANGE to 499
+ * 06/20/02 RD	replaced deprecated header <new.h> with <new>
+ * 05/27/02 RD	updated error ranges for current tool set
+ * 01/30/02 RD	increased range of SIR error codes (SIR_ERROR_RANGE)
+ * 01/03/02 RD	updated copyright range to 2002
+ * 11/20/01 RD	added support for 'new'/'delete' exceptions
+ *		(conditional compilation with GL_MEM_USE_EXCEPTIONS)
+ * 11/15/01 RD	added inclusion of GL_System.h
+ * 11/02/01 RD	moved IO support functions into separate file GL_FileIO.h
+ * 08/08/01 RD	adjusted version to 1.2
+ * 08/06/01 RD	adjusted version to 1.1
  * 05/26/01 RD	eliminated support for binary SIR files (import/export)
  * 05/25/01 RD	removed code not needed for the SCRC
  * 05/24/01 RD	added copyright string definition
@@ -23,19 +44,17 @@
 #define GLOBAL_H
 
 #include <stdio.h>
-#include <new.h>
+#include <new>
 #include "GL_String.h"
+#include "GL_FileIO.h"
+#include "GL_System.h"
 #include "longlong.h"
 
 
 /*** type definitions ***************************************************/
 
 
-typedef bool		BOOL;
-typedef int		ERROR;
-typedef double		FLOAT;	/* use double precision floating point */
-
-typedef String          string; /* use our own Strings */
+typedef int			ERROR;
 
 #ifdef HAVE_LLONG
 /* use the built-in "long long" */
@@ -66,9 +85,9 @@ typedef UNSIGNED_LONG_LONG	TIME;	/* time is unsigned long long */
 	/* System's name, version number, and copyright */
 
 #define GL_SPECC_NAME			"SCRC"
-#define GL_SPECC_VERSION_STRING		"1.1"
+#define GL_SPECC_VERSION_STRING		"2.1"
 #define GL_SPECC_COPYRIGHT_STRING	\
-		"(c) 1997-2001 CECS, University of California, Irvine"
+		"(c) 1997-2006 CECS, University of California, Irvine"
 
 
 	/* SpecC language definitions */
@@ -86,16 +105,6 @@ typedef UNSIGNED_LONG_LONG	TIME;	/* time is unsigned long long */
 #define ABS(a)		((a) > (0) ? (a) : (-(a)))
 
 
-	/* boolean values */
-
-#ifndef FALSE
-#define FALSE		false
-#endif /* FALSE */
-#ifndef TRUE
-#define TRUE		true
-#endif /* TRUE */
-
-
 	/* result values at program termination */
 
 #define RESULT_OK	 0	/* everything ok */
@@ -105,13 +114,16 @@ typedef UNSIGNED_LONG_LONG	TIME;	/* time is unsigned long long */
 	/* memory management */
 
 #define GL_MEM_NUMBER_RETRYS	5	/* number of malloc() retrys */
+#ifdef __STRICT_ANSI__
+#define GL_MEM_USE_EXCEPTIONS		/* 'new' and 'delete' do 'throw' */
+#endif /* __STRICT_ANSI__ */
 
 
 	/* floating point precision */
 
 #define GL_FLOAT_OUT_FMT	"% 16.6f"	/* format string for printf() */
 #define GL_FLOAT_IN_FMT		"%lf"		/* format string for scanf() */
-#define GL_EPSILON	((FLOAT)0.000001)	/* precision for comparisons */
+#define GL_EPSILON	((double)0.000001)	/* precision for comparisons */
 
 
 	/* common warning levels */
@@ -129,15 +141,15 @@ typedef UNSIGNED_LONG_LONG	TIME;	/* time is unsigned long long */
 #define FLAG_YES	1	/* return value reserved for "yes" */
 #define FLAG_NO		0	/* return value reserved for "no" */
 
-#define GL_ERROR_BASE	1000	/* Global	(R.D.) */
+#define GL_ERROR_BASE	1000	/* global		libGlobal */
 #define GL_ERROR_RANGE	  99
-#define SIR_ERROR_BASE	2000	/* IntRep	(R.D.) */
-#define SIR_ERROR_RANGE	 199
-#define CCD_ERROR_BASE	3000	/* CcDriver	(R.D.) */
+#define SIR_ERROR_BASE	2000	/* int. representation	libIntRep */
+#define SIR_ERROR_RANGE	 499
+#define CCD_ERROR_BASE	3000	/* compiler driver	libCcDriver */
 #define CCD_ERROR_RANGE	  99
-#define PRS_ERROR_BASE	3100	/* Parser	(R.D.) */
+#define PRS_ERROR_BASE	3100	/* SpecC parser		libParser */
 #define PRS_ERROR_RANGE	  99
-#define SCC_ERROR_BASE	3200	/* Compiler	(R.D.) */
+#define SCC_ERROR_BASE	3200	/* SpecC compiler	scc */
 #define SCC_ERROR_RANGE	  99
 
 
@@ -164,25 +176,22 @@ typedef UNSIGNED_LONG_LONG	TIME;	/* time is unsigned long long */
 #define GL_ERROR_NOT_A_LEGAL_IDENTIFIER_3	(GL_ERROR_BASE + 12)
 #define GL_ERROR_NOT_A_LEGAL_FILENAME		(GL_ERROR_BASE + 13)
 #define GL_ERROR_NOT_A_LEGAL_FILE_PATH		(GL_ERROR_BASE + 14)
+#define GL_ERROR_TMP_FILE_CREATE_FAILED		(GL_ERROR_BASE + 15)
+#define GL_ERROR_TMP_FILE_REWIND_FAILED		(GL_ERROR_BASE + 16)
 #define GL_ERROR_FILE_DELETE_FAILED		(GL_ERROR_BASE + 17)
-
-
-/*** exported type declarations *****************************************/
-
-
-typedef struct GL_StringList	GL_STRINGLIST;
-
-struct GL_StringList
-{
-GL_STRINGLIST	*Succ;
-const char	*String;
-};
+#define GL_ERROR_ILLEGAL_ARGUMENTS		(GL_ERROR_BASE + 18)
+#define GL_ERROR_RW_MODE_VIOLATION		(GL_ERROR_BASE + 19)
+#define GL_ERROR_FILE_RW_OPEN_FAILED		(GL_ERROR_BASE + 20)
+#define GL_ERROR_FILE_IO_ERROR			(GL_ERROR_BASE + 21)
+#define GL_ERROR_NOT_AT_END_OF_STRING		(GL_ERROR_BASE + 22)
+#define GL_ERROR_FILE_NOT_WRITABLE		(GL_ERROR_BASE + 23)
 
 
 /*** exported variables *************************************************/
 
 
-extern ERROR	GL_Error;		/* last error of this module */
+extern ERROR		GL_Error;	/* last error of this module */
+extern gl_string	GL_ErrorMessage;
 
 extern int	GL_VerbosityLevel;	/* (published by main module) */
 extern int	GL_WarningLevel;	/* (published by main module) */
@@ -204,7 +213,7 @@ extern void GL_Reset(void);
 	/* error code to error message conversion */
 
 extern const char *GL_ErrorText(
-	ERROR	ErrorNumber);
+	ERROR		ErrorNumber);
 
 
 	/* determine error message from the OS (errno, etc.) */
@@ -239,16 +248,32 @@ extern void GL_PrintWarningFmt(
 	/* overloaded C++ memory management */
 
 extern void *operator new (
-	size_t		NumBytes);
+	size_t	NumBytes)
+#ifdef GL_MEM_USE_EXCEPTIONS
+	throw (std::bad_alloc)
+#endif /* GL_MEM_USE_EXCEPTIONS */
+	;
 
 extern void *operator new[] (
-	size_t		NumBytes);
+	size_t		NumBytes)
+#ifdef GL_MEM_USE_EXCEPTIONS
+	throw (std::bad_alloc)
+#endif /* GL_MEM_USE_EXCEPTIONS */
+	;
 
 extern void operator delete (
-	void		*MemAddress);
+	void		*MemAddress)
+#ifdef GL_MEM_USE_EXCEPTIONS
+	throw ( )
+#endif /* GL_MEM_USE_EXCEPTIONS */
+	;
 
 extern void operator delete[] (
-	void		*MemAddress);
+	void		*MemAddress)
+#ifdef GL_MEM_USE_EXCEPTIONS
+	throw ( )
+#endif /* GL_MEM_USE_EXCEPTIONS */
+	;
 
 
 	/* smarter versions of malloc() and free() */
@@ -290,41 +315,6 @@ extern const char *GL_GetBaseName(	/* obtains the base name from a path */
 	const char	*Path);		/* (file name without suffix) */
 
 
-/*** file I/O ***********************************************************/
-
-
-	/* support functions for file I/O */
-
-extern FILE *GL_OpenFileForReading(	/* open a file for read access */
-	const char	*Filename);	/* (supports "-" for stdin) */
-
-extern FILE *GL_OpenFileForWriting(	/* open a (new) file for write access */
-	const char	*Filename);	/* (supports "-" for stdout) */
-
-extern void GL_CloseFile(		/* close a file */
-	FILE		*FileHandle);
-
-extern ERROR GL_CloseFileWithCheck(	/* close a file and check for errors */
-	FILE		*FileHandle);
-
-extern ERROR GL_DeleteFile(		/* delete a file */
-	const char	*Filename);
-
-extern ERROR GL_FileIsReadable(		/* check if file is read-accessable */
-	const char	*Filename);	/* (supports "-" for stdin) */
-
-extern ERROR GL_FileIsCreatable(	/* check if file is write-accessable */
-	const char	*Filename);	/* (supports "-" for stdout) */
-
-extern ERROR GL_ReadEOF(		/* read (and check) EOF from file */
-	FILE		*FileHandle);
-
-extern const char *GL_FindFileInDirList(/* find a file for read access */
-	const char	*Filename,	/* (applying DirList and Suffix) */
-	const char	*Suffix = NULL,	/* (returns NULL if not found) */
-	GL_STRINGLIST	*DirList = NULL);
-
-
 #endif /* GLOBAL_H */
 
 /* EOF Global.h */
