@@ -4,10 +4,16 @@
 /* Author: Rainer Doemer			first version: 08/31/97 */
 /************************************************************************/
 
-/* last update: 05/25/01 */
+/* last update: 09/26/06 */
 
 /* modifications: (most recent first)
  *
+ * 09/26/06 PC  Adjustments for scrc 2.1
+ * 06/03/05 RD	reorganized and renamed global type names
+ * 06/15/04 PC  Adjustments for scrc 2.0
+ * 11/21/02 RD	modified call format strings to have exactly one option marker
+ * 11/19/02 RD	added quoting of user-supplied arguments (see Quote())
+ * 11/21/01 RD	took out default arguments from function definitions
  * 05/25/01 RD	eliminated support for automatic IP wrapping (scc -ip option)
  * 04/30/01 RD	replaced use of obsolete form() from "stream.h" with own one
  * 04/30/01 RD	added this header (last change was 06/18/00)
@@ -18,6 +24,7 @@
 #include <stdlib.h>
 #include <strings.h>
 #include <assert.h>
+#include <ctype.h>
 
 
 /*** constants and macros ***********************************************/
@@ -50,6 +57,9 @@ static ERROR CheckCallFormat(
 	const char	*OptionMarker,
 	const char	*Tool);
 
+static const char *Quote(
+        const char      *Argument);
+
 static const char *BuildCommandLine(
 	const char	*Call,
 	const char	*InputFilename,
@@ -59,13 +69,13 @@ static const char *BuildCommandLine(
 	CCD_OPTLIST	*OptionList,
 	const char	*Flags0 = NULL,
 	const char	*List1Flag = NULL,
-	GL_STRINGLIST	*List1 = NULL,
+	gl_stringlist	*List1 = NULL,
 	const char	*Flags1 = NULL,
 	const char	*List2Flag = NULL,
-	GL_STRINGLIST	*List2 = NULL,
+	gl_stringlist	*List2 = NULL,
 	const char	*Flags2 = NULL,
 	const char	*List3Flag = NULL,
-	GL_STRINGLIST	*List3 = NULL,
+	gl_stringlist	*List3 = NULL,
 	const char	*Flags3 = NULL);
 
 static ERROR ExecuteCommand(
@@ -82,7 +92,7 @@ ERROR	CCD_Error = CCD_ERROR_NO_ERROR;	/* last error of this module */
 /*** internal variables *************************************************/
 
 
-static string	ErrorMessage;
+static gl_string	ErrorMessage;
 
 
 static const char	*CallPP = CCD_DEFAULT_PP_CALL;
@@ -110,12 +120,13 @@ static CCD_OPTLIST	*OptionsCC = &DefaultOptionsCC[0];
 static CCD_OPTLIST	*OptionsLD = &DefaultOptionsLD[0];
 
 static CCD_MODE		CompilerMode = CCD_MODE_STANDARD;
+static CCD_TARGET       CompilerTarget = CCD_TARGET_EXECUTABLE;
 
-static GL_STRINGLIST	*Definitions = NULL;
-static GL_STRINGLIST	*UnDefs = NULL;
-static GL_STRINGLIST	*IncludePath = NULL;
-static GL_STRINGLIST	*LibraryPath = NULL;
-static GL_STRINGLIST	*LibraryList = NULL;
+static gl_stringlist	*Definitions = NULL;
+static gl_stringlist	*UnDefs = NULL;
+static gl_stringlist	*IncludePath = NULL;
+static gl_stringlist	*LibraryPath = NULL;
+static gl_stringlist	*LibraryList = NULL;
 
 static const char	*StdMacroDefs	= CCD_STD_DEFINES;
 static const char	*StdUnDefs	= CCD_STD_UNDEFINES;
@@ -172,21 +183,17 @@ switch (ErrorNumber)
      case CCD_ERROR_EXACTLY_ONE_I_REQUIRED:
 	  return(ErrorMessage.chars()
 		/* "Bad format string for %s call:" GL_ERROR_MSG_NEWLINE
-		"exactly one %%i escape sequence required" */);
+		"exactly one %%i marker required" */);
      case CCD_ERROR_EXACTLY_ONE_O_REQUIRED:
 	  return(ErrorMessage.chars()
 		/* "Bad format string for %s call:" GL_ERROR_MSG_NEWLINE
 		"exactly one %%o escape sequence required" */);
-     case CCD_ERROR_MIN_ONE_FLAG_REQUIRED:
+     case CCD_ERROR_EXACTLY_ONE_FLAG_REQUIRED:
 	  return(ErrorMessage.chars()
 		/* "Bad format string for %s call:" GL_ERROR_MSG_NEWLINE
-		"at least one %s escape sequence required" */);
-     case CCD_ERROR_UNKNOWN_ESCAPE_SEQUENCE:
-	  return(ErrorMessage.chars()
-		/* "Bad format string for %s call:" GL_ERROR_MSG_NEWLINE
-		"only %%i, %%o, %s escape sequences allowed" */);
+		"exactly one %s marker required" */);
      case CCD_ERROR_PERCENT_CHAR_NOT_ALLOWED:
-	  return("Percent character (%) not allowed in compiler options");
+	  return("Percent character (%) not allowed in file name");
     } /* hctiws */
 
 return("Unknown error number (internal)");
@@ -235,6 +242,7 @@ if (OptionsLD != &DefaultOptionsLD[0])
     } /* fi */
 
 CompilerMode = CCD_MODE_STANDARD;
+CompilerTarget = CCD_TARGET_EXECUTABLE;
 
 Definitions = NULL;
 UnDefs = NULL;
@@ -270,6 +278,24 @@ return(CCD_ERROR_NO_ERROR);
 
 } /* end of CCD_SetCompilerMode */
 
+ERROR CCD_SetTarget(CCD_TARGET Target)
+{
+
+switch(Target)
+   { case CCD_TARGET_EXECUTABLE:
+     case CCD_TARGET_SHARED_LIB:
+        { CompilerTarget = Target;
+          break;
+         }
+     default:
+        { return(CCD_ERROR_ILLEGAL_TARGET);
+         }
+    } /* hctiws */
+
+return(CCD_ERROR_NO_ERROR);
+
+} /* end of CCD_SetTarget */
+
 
 ERROR CCD_SetPreprocessorCall(const char *FormatString)
 {
@@ -321,15 +347,11 @@ ERROR CCD_SetPreprocessorOption(const char *Option)
 CCD_OPTLIST	*NewOpt,
 		*OptPred;
 
-if (index(Option, '%'))
-   { return(CCD_ERROR_PERCENT_CHAR_NOT_ALLOWED);
-    } /* fi */
-
 if (!(NewOpt = new(CCD_OPTLIST)))
    { return(CCD_ERROR_OUT_OF_MEMORY);
     } /* fi */
 NewOpt->Succ = NULL;
-NewOpt->Option = Option;
+NewOpt->Option = Quote(Option);         /* pre-quote the option */
 
 if (OptionsPP == &DefaultOptionsPP[0])	/* still default options? */
    { OptionsPP = NewOpt;		/* start user options list */
@@ -352,15 +374,11 @@ ERROR CCD_SetCompilerOption(const char *Option)
 CCD_OPTLIST	*NewOpt,
 		*OptPred;
 
-if (index(Option, '%'))
-   { return(CCD_ERROR_PERCENT_CHAR_NOT_ALLOWED);
-    } /* fi */
-
 if (!(NewOpt = new(CCD_OPTLIST)))
    { return(CCD_ERROR_OUT_OF_MEMORY);
     } /* fi */
 NewOpt->Succ = NULL;
-NewOpt->Option = Option;
+NewOpt->Option = Quote(Option);         /* pre-quote the option */
 
 if (OptionsCC == &DefaultOptionsCC[0])	/* still default options? */
    { OptionsCC = NewOpt;		/* start user options list */
@@ -383,15 +401,11 @@ ERROR CCD_SetLinkerOption(const char *Option)
 CCD_OPTLIST	*NewOpt,
 		*OptPred;
 
-if (index(Option, '%'))
-   { return(CCD_ERROR_PERCENT_CHAR_NOT_ALLOWED);
-    } /* fi */
-
 if (!(NewOpt = new(CCD_OPTLIST)))
    { return(CCD_ERROR_OUT_OF_MEMORY);
     } /* fi */
 NewOpt->Succ = NULL;
-NewOpt->Option = Option;
+NewOpt->Option = Quote(Option);         /* pre-quote the option */
 
 if (OptionsLD == &DefaultOptionsLD[0])	/* still default options? */
    { OptionsLD = NewOpt;		/* start user options list */
@@ -410,7 +424,7 @@ return(CCD_ERROR_NO_ERROR);
 
 
 ERROR CCD_SetMacroDefs(			/* specify the macro definitions */
-	GL_STRINGLIST	*DList)
+	gl_stringlist	*DList)
 {
 
 Definitions = DList;
@@ -421,7 +435,7 @@ return(CCD_ERROR_NO_ERROR);
 
 
 ERROR CCD_SetUnDefs(			/* specify the un-definitions */
-	GL_STRINGLIST	*UList)
+	gl_stringlist	*UList)
 {
 
 UnDefs = UList;
@@ -432,7 +446,7 @@ return(CCD_ERROR_NO_ERROR);
 
 
 ERROR CCD_SetIncludePath(		/* specify the include path */
-	GL_STRINGLIST	*IList)
+	gl_stringlist	*IList)
 {
 
 IncludePath = IList;
@@ -443,7 +457,7 @@ return(CCD_ERROR_NO_ERROR);
 
 
 ERROR CCD_SetLibraryPath(		/* specify the library path */
-	GL_STRINGLIST	*LList)
+	gl_stringlist	*LList)
 {
 
 LibraryPath = LList;
@@ -454,7 +468,7 @@ return(CCD_ERROR_NO_ERROR);
 
 
 ERROR CCD_SetLibraries(			/* specify the library list */
-	GL_STRINGLIST	*lList)
+	gl_stringlist	*lList)
 {
 
 LibraryList = lList;
@@ -523,6 +537,16 @@ ERROR CCD_Preprocessor(
 {
 const char	*CommandLine;
 
+assert(InputFilename != NULL);
+assert(OutputFilename != NULL);
+
+if (index(InputFilename, '%'))
+   { return(CCD_ERROR_PERCENT_CHAR_NOT_ALLOWED);
+    } /* fi */
+if (index(OutputFilename, '%'))
+   { return(CCD_ERROR_PERCENT_CHAR_NOT_ALLOWED);
+    } /* fi */
+
 CommandLine = BuildCommandLine(CallPP,
 			InputFilename, OutputFilename,
 			"%p", "preprocessor", OptionsPP,
@@ -557,13 +581,23 @@ ERROR CCD_Compiler(
 {
 const char	*CommandLine;
 
+assert(InputFilename != NULL);
+assert(OutputFilename != NULL);
+
+if (index(InputFilename, '%'))
+   { return(CCD_ERROR_PERCENT_CHAR_NOT_ALLOWED);
+    } /* fi */
+if (index(OutputFilename, '%'))
+   { return(CCD_ERROR_PERCENT_CHAR_NOT_ALLOWED);
+    } /* fi */
+
 CommandLine = BuildCommandLine(CallCC,
-			InputFilename, OutputFilename,
-			"%c", "compiler", OptionsCC,
-			NULL,
-			CCD_DEFINITION_OPT, Definitions, StdMacroDefs,
-			CCD_UNDEF_OPT, UnDefs, StdUnDefs,
-			CCD_INCLUDE_PATH_OPT, IncludePath, StdIncludePath);
+		InputFilename, OutputFilename,
+		"%c", "compiler", OptionsCC,
+		NULL,
+		CCD_DEFINITION_OPT, Definitions, StdMacroDefs,
+		CCD_UNDEF_OPT, UnDefs, StdUnDefs,
+		CCD_INCLUDE_PATH_OPT, IncludePath, StdIncludePath);
 
 if ((CCD_Error = GL_FileIsReadable(InputFilename)))
    { return(CCD_Error);
@@ -591,12 +625,22 @@ ERROR CCD_Linker(
 {
 const char	*CommandLine;
 
+assert(InputFilename != NULL);
+assert(OutputFilename != NULL);
+
+if (index(InputFilename, '%'))
+   { return(CCD_ERROR_PERCENT_CHAR_NOT_ALLOWED);
+    } /* fi */
+if (index(OutputFilename, '%'))
+   { return(CCD_ERROR_PERCENT_CHAR_NOT_ALLOWED);
+    } /* fi */
+
 CommandLine = BuildCommandLine(CallLD,
-			InputFilename, OutputFilename,
-			"%l", "linker", OptionsLD,
-			NULL,
-			CCD_LIBRARY_PATH_OPT, LibraryPath, StdLibraryPath,
-			CCD_LIBRARY_LINK_OPT, LibraryList, StdLibraries);
+		InputFilename, OutputFilename,
+		"%l", "linker", OptionsLD,
+		NULL,
+		CCD_LIBRARY_PATH_OPT, LibraryPath, StdLibraryPath,
+		CCD_LIBRARY_LINK_OPT, LibraryList, StdLibraries);
 
 if ((CCD_Error = GL_FileIsReadable(InputFilename)))
    { return(CCD_Error);
@@ -628,36 +672,29 @@ static ERROR CheckCallFormat(
 	const char	*OptionMarker,
 	const char	*Tool)
 {
-string	TmpString;
+gl_string	TmpString;
 
 TmpString = FormatString;
 if (TmpString.freq("%i") != 1)
    { ErrorMessage.form(
 	"Bad format string for %s call:" GL_ERROR_MSG_NEWLINE
-	"exactly one %%i escape sequence required",
+	"exactly one %%i marker required",
 	Tool);
      return(CCD_ERROR_EXACTLY_ONE_I_REQUIRED);
     } /* fi */
 if (TmpString.freq("%o") != 1)
    { ErrorMessage.form(
 	"Bad format string for %s call:" GL_ERROR_MSG_NEWLINE
-	"exactly one %%o escape sequence required",
+	"exactly one %%o marker required",
 	Tool);
      return(CCD_ERROR_EXACTLY_ONE_O_REQUIRED);
     } /* fi */
 if (TmpString.freq(OptionMarker) < 1)
    { ErrorMessage.form(
 	"Bad format string for %s call:" GL_ERROR_MSG_NEWLINE
-	"at least one %s escape sequence required",
+	"exactly one %s marker required",
 	Tool, OptionMarker);
-     return(CCD_ERROR_MIN_ONE_FLAG_REQUIRED);
-    } /* fi */
-if (TmpString.freq(OptionMarker) + 2 != TmpString.freq("%"))
-   { ErrorMessage.form(
-	"Bad format string for %s call:" GL_ERROR_MSG_NEWLINE
-	"only %%i, %%o, %s escape sequences allowed",
-	Tool, OptionMarker);
-     return(CCD_ERROR_UNKNOWN_ESCAPE_SEQUENCE);
+     return(CCD_ERROR_EXACTLY_ONE_FLAG_REQUIRED);
     } /* fi */
 
 return(CCD_ERROR_NO_ERROR);
@@ -665,6 +702,57 @@ return(CCD_ERROR_NO_ERROR);
 } /* end of CheckCallFormat */
 
 
+static const char *Quote(
+        const char      *Argument)
+{
+const char	*CharPtr;
+bool		QuotingNeeded;
+static gl_string QuotedArg;
+
+assert(Argument != NULL);
+
+if (*Argument == 0)
+   { return("\"\"");    /* explicit empty argument */
+    } /* fi */
+
+QuotingNeeded = false;
+CharPtr = Argument;
+while(*CharPtr)
+   { if (  (!(isalnum(*CharPtr)))
+	 &&(*CharPtr != '+')
+	 &&(*CharPtr != '-')
+	 &&(*CharPtr != '.')
+	 &&(*CharPtr != '_')
+	 &&(*CharPtr != '=')
+	 &&(*CharPtr != '/'))
+	{ QuotingNeeded = true;
+	  break;
+	 } /* fi */
+     CharPtr++;
+    } /* elihw */
+
+if (! QuotingNeeded)
+   { return(Argument);  /* no quoting needed */
+    } /* fi */
+
+QuotedArg = '"';
+CharPtr = Argument;
+while(*CharPtr)
+   { if (  (*CharPtr == '\\')   // escape the backslash
+         ||(*CharPtr == '`')    // escape the left-tick
+         ||(*CharPtr == '"')    // escape the double-quote
+         ||(*CharPtr == '$'))   // escape the dollar
+        { QuotedArg += '\\';
+         } /* fi */
+     QuotedArg += *CharPtr++;
+    } /* elihw */
+QuotedArg += '"';
+
+return(QuotedArg.chars());
+
+} /* end of Quote */
+
+
 static const char *BuildCommandLine(
 	const char	*Call,
 	const char	*InputFilename,
@@ -672,23 +760,23 @@ static const char *BuildCommandLine(
 	const char	*OptionMarker,
 	const char	*Tool,
 	CCD_OPTLIST	*OptionList,
-	const char	*Flags0 = NULL,
-	const char	*List1Flag = NULL,
-	GL_STRINGLIST	*List1 = NULL,
-	const char	*Flags1 = NULL,
-	const char	*List2Flag = NULL,
-	GL_STRINGLIST	*List2 = NULL,
-	const char	*Flags2 = NULL,
-	const char	*List3Flag = NULL,
-	GL_STRINGLIST	*List3 = NULL,
-	const char	*Flags3 = NULL)
+	const char	*Flags0 /* = NULL */,
+	const char	*List1Flag /* = NULL */,
+	gl_stringlist	*List1 /* = NULL */,
+	const char	*Flags1 /* = NULL */,
+	const char	*List2Flag /* = NULL */,
+	gl_stringlist	*List2 /* = NULL */,
+	const char	*Flags2 /* = NULL */,
+	const char	*List3Flag /* = NULL */,
+	gl_stringlist	*List3 /* = NULL */,
+	const char	*Flags3 /* = NULL */)
 {
-static string	CommandLine;
+static gl_string CommandLine;
 CCD_OPTLIST	*Option;
-GL_STRINGLIST	*StringPtr;
-int		NumOptions,
-		i;
-string		Flags;
+gl_stringlist	*StringPtr;
+gl_string	Flags,
+		Flag,
+		Options;
 
 #ifdef CCD_DEBUG
 printf("CCD_DEBUG: Call   = \"%s\"\n", Call);
@@ -699,146 +787,148 @@ printf("CCD_DEBUG: Output = \"%s\"\n", OutputFilename);
 CommandLine = Call;			/* start with the format string */
 
 assert(CommandLine.freq("%i") == 1);	/* replace the '%i' with input */
-CommandLine.at("%i") = InputFilename;
+CommandLine.at("%i") = Quote(InputFilename);
 
 assert(CommandLine.freq("%o") == 1);	/* replace the '%o' with output */
-CommandLine.at("%o") = OutputFilename;
+CommandLine.at("%o") = Quote(OutputFilename);
 
-Flags = OptionMarker;			/* handle the flags */
+Flags = "";				/* handle the flags */
 switch(CompilerMode)
    { case CCD_MODE_STANDARD:
-	{ /* no flag */
-	  break;
-	 }
+        { /* no flag */
+          break;
+         }
      case CCD_MODE_DEBUG:
-	{ Flags += " " CCD_DEBUG_OPT;
-	  break;
-	 }
+        { Flags += " " CCD_DEBUG_OPT;           // hard-quoted
+          break;
+         }
      case CCD_MODE_OPTIMIZE:
-	{ Flags += " " CCD_OPTIMIZE_OPT;
-	  break;
-	 }
+        { Flags += " " CCD_OPTIMIZE_OPT;        // hard-quoted
+          break;
+         }
      default:
-	{ assert(FALSE);	/* unknown compiler mode */
+	{ assert(false);	/* unknown compiler mode */
 	 }
     } /* hctiws */
 
 switch(GL_WarningLevel)
    { case GL_WARN_ALWAYS:
-	{ Flags += " " CCD_NO_WARN_OPT;
-	  break;
-	 }
+        { Flags += " " CCD_NO_WARN_OPT;         // hard-quoted
+          break;
+         }
      case GL_WARN_IMPORTANT:
-	{ Flags += " " CCD_IMPORTANT_WARN_OPT;
-	  break;
-	 }
+        { Flags += " " CCD_IMPORTANT_WARN_OPT;  // hard-quoted
+          break;
+         }
      case GL_WARN_STANDARD:
      case GL_WARN_INFORMATIVE:
-	{ Flags += " " CCD_STANDARD_WARN_OPT;
-	  break;
-	 }
+        { Flags += " " CCD_STANDARD_WARN_OPT;   // hard-quoted
+          break;
+         }
      case GL_WARN_PEDANTIC:
      default:
-	{ Flags += " " CCD_PEDANTIC_WARN_OPT;
-	  break;
-	 }
+        { Flags += " " CCD_PEDANTIC_WARN_OPT;   // hard-quoted
+          break;
+         }
     } /* hctiws */
 
 if (Flags0)
    { Flags += " ";
-     Flags += Flags0;
+     Flags += Flags0;   // hard-quoted
     } /* fi */
 
 StringPtr = List1;
 while(StringPtr)
-   { Flags += " ";
-     if (List1Flag)
-	{ Flags += List1Flag;
-	 } /* fi */
-     Flags += StringPtr->String;
+   { if (List1Flag)
+        { Flag = List1Flag;
+          Flag += StringPtr->String;
+         } /* fi */
+     else
+        { Flag = StringPtr->String;
+         } /* esle */
+     Flags += " ";
+     Flags += Quote(Flag.chars());
      StringPtr = StringPtr->Succ;
     } /* elihw */
 
 if (Flags1)
    { Flags += " ";
-     Flags += Flags1;
+     Flags += Flags1;   // hard-quoted
     } /* fi */
 
 StringPtr = List2;
 while(StringPtr)
-   { Flags += " ";
-     if (List2Flag)
-	{ Flags += List2Flag;
-	 } /* fi */
-     Flags += StringPtr->String;
+   { if (List2Flag)
+        { Flag = List2Flag;
+          Flag += StringPtr->String;
+         } /* fi */
+     else
+        { Flag = StringPtr->String;
+         } /* esle */
+     Flags += " ";
+     Flags += Quote(Flag.chars());
      StringPtr = StringPtr->Succ;
     } /* elihw */
 
 if (Flags2)
    { Flags += " ";
-     Flags += Flags2;
+     Flags += Flags2;   // hard-quoted
     } /* fi */
 
 StringPtr = List3;
 while(StringPtr)
-   { Flags += " ";
-     if (List3Flag)
-	{ Flags += List3Flag;
-	 } /* fi */
-     Flags += StringPtr->String;
+   { if (List3Flag)
+        { Flag = List3Flag;
+          Flag += StringPtr->String;
+         } /* fi */
+     else
+        { Flag = StringPtr->String;
+         } /* esle */
+     Flags += " ";
+     Flags += Quote(Flag.chars());
      StringPtr = StringPtr->Succ;
     } /* elihw */
 
 if (Flags3)
    { Flags += " ";
-     Flags += Flags3;
+     Flags += Flags3;   // hard-quoted
     } /* fi */
 
 #ifdef CCD_DEBUG
 printf("CCD_DEBUG: Flags = \"%s\"\n", Flags.chars());
 #endif /* CCD_DEBUG */
-assert(CommandLine.freq(OptionMarker) >= 1);
-CommandLine.at(OptionMarker) = Flags;
 
-NumOptions = CommandLine.freq(OptionMarker);	/* handle the options */
-assert(NumOptions >= 1);
-i = 0;
-Option = (CCD_OPTLIST*) &OptionList; /* (predecessor) */
-while ((Option = Option->Succ))
-   {
+Options = "";                                   /* handle the options */
+Option = OptionList;
+while(Option)
+   { if (  (Option->Option)
+         &&(*Option->Option))
+        {
 #ifdef CCD_DEBUG
-printf("CCD_DEBUG: Option = \"%s\"\n", Option->Option);
+          printf("CCD_DEBUG: Option = \"%s\"\n", Option->Option);
 #endif /* CCD_DEBUG */
-     if (i >= NumOptions)
-	{ if (*Option->Option)
-	     { CommandLine += " ";
-	       CommandLine += Option->Option;	/* append the option */
-	      } /* fi */
-	 } /* fi */
-     else
-	{ CommandLine.at(OptionMarker)
-			= Option->Option;	/* insert the option */
-	  i++;
-	 } /* esle */
+          Options += " ";
+          Options += Option->Option;    // hard- or pre-quoted
+         } /* fi */
+     Option = Option->Succ;
     } /* elihw */
 
-if (i < NumOptions)
-   { GL_PrintWarningFmt(GL_WARN_STANDARD,
-			"Excessive option escapes (%s) in %s call ignored",
-			OptionMarker, Tool);
-     do { CommandLine.del(OptionMarker);
-	 } while(++i < NumOptions);
-    } /* fi */
+#ifdef CCD_DEBUG
+printf("CCD_DEBUG: Options = \"%s\"\n", Options.chars());
+#endif /* CCD_DEBUG */
+
+assert(CommandLine.freq(OptionMarker) == 1);    /* replace the option marker */
+Flags += Options;                               /* with flags and options    */
+CommandLine.at(OptionMarker) = Flags;
+
 #ifdef CCD_DEBUG
 printf("CCD_DEBUG: CommandLine = \"%s\"\n", CommandLine.chars());
 #endif /* CCD_DEBUG */
-assert(CommandLine.freq("%") == 0);
 
-return(CommandLine.chars());	/* return the composed command line */
+return(CommandLine.chars());    /* return the composed command line */
 
 } /* end of BuildCommandLine */
 
-
 static ERROR ExecuteCommand(
 	const char	*CommandLine,
 	const char	*Meaning)
@@ -948,10 +1038,6 @@ if ((Error = CCD_SetLinkerCall("gcc %i -o %o %l")))
    { GL_PrintError(Error, CCD_ErrorText(Error));
      exit(RESULT_ERROR);
     } /* fi */
-if ((Error = CCD_SetLinkerOption("")))	/* (redundant) */
-   { GL_PrintError(Error, CCD_ErrorText(Error));
-     exit(RESULT_ERROR);
-    } /* fi */
 
 puts("\nLinker(\"HelloWorld.o\", \"HelloWorld\")...");
 #ifdef GNUWIN32
