@@ -4,11 +4,12 @@
 /* Author: Rainer Doemer			first version: 08/31/97 */
 /************************************************************************/
 
-
-/* last update: 06/15/04 */
+/* last update: 09/26/06 */
 
 /* modifications: (most recent first)
  *
+ * 09/26/06 PC  Adjustments for scrc 2.1
+ * 06/06/05 RD	reorganized and renamed global type names
  * 06/15/04 PC  Adjustments for scrc 2.0
  * 05/29/03 RD	get definitions for standard import directory from CcDriver.h
  * 11/03/02 RD	fixed clearing of settings lists (after code review by AG)
@@ -79,58 +80,58 @@ static ERROR GetCommand(		/* parse the command argument */
 
 
 static ERROR	SCC_Error = SCC_ERROR_NO_ERROR;	/* last error of this module */
-static string	ErrorMessage;
+static gl_string ErrorMessage;
 
 
 	/*** command line options and derivations (and their defaults) ***/
 
-static string		DesignName;	/* no default */
+static gl_string	DesignName;	/* no default */
 
-static string		FilenameSCin;	/* these will be build */
-static string		FilenameSCout;	/* based on DesignName */
-static string		FilenameSI;
-static string		FilenameCC;
-static string		FilenameH;
-static string		FilenameO;
-static string		FilenameOUT;
+static gl_string	FilenameSCin;	/* these will be build */
+static gl_string	FilenameSCout;	/* based on DesignName */
+static gl_string	FilenameSI;
+static gl_string	FilenameCC;
+static gl_string	FilenameH;
+static gl_string	FilenameO;
+static gl_string	FilenameOUT;
 
-static BOOL		CallPreprocessor	= TRUE;	/* based on command */
-static BOOL		CallParser		= TRUE;
-static BOOL		CallDeparser		= FALSE;
-static BOOL		CallTranslator		= TRUE;
-static BOOL		CallCompiler		= TRUE;
-static BOOL		CallLinker		= TRUE;
+static bool		CallPreprocessor	= true;	/* based on command */
+static bool		CallParser		= true;
+static bool		CallDeparser		= false;
+static bool		CallTranslator		= true;
+static bool		CallCompiler		= true;
+static bool		CallLinker		= true;
 
-static string		*InputFilename  = &FilenameSCin;/* based on command */
-static string		*OutputFilename = &FilenameOUT;
+static gl_string	*InputFilename  = &FilenameSCin;/* based on command */
+static gl_string	*OutputFilename = &FilenameOUT;
 
-static GL_STRINGLIST	*PDirList = NULL,
+static gl_stringlist	*PDirList = NULL,
 			*PDirListInsert = NULL;
-static GL_STRINGLIST	*DDefList = NULL,
+static gl_stringlist	*DDefList = NULL,
 			*DDefListLast = NULL;
-static GL_STRINGLIST	*UDefList = NULL,
+static gl_stringlist	*UDefList = NULL,
 			*UDefListLast = NULL;
-static GL_STRINGLIST	*IDirList = NULL,
+static gl_stringlist	*IDirList = NULL,
 			*IDirListLast = NULL;
-static GL_STRINGLIST	*LDirList = NULL,
+static gl_stringlist	*LDirList = NULL,
 			*LDirListLast = NULL;
-static GL_STRINGLIST	*lLibList = NULL,
+static gl_stringlist	*lLibList = NULL,
 			*lLibListLast = NULL;
 
-static BOOL		UsageInfoOnly	= FALSE;
+static bool		UsageInfoOnly	= false;
 static int		VerbosityLevel	= 0;
 static int		WarningLevel	= 0;
-static BOOL		DebugMode	= FALSE;
-static BOOL		OptimizeMode	= FALSE;
-static BOOL		GenerateLines	= TRUE;
-static BOOL		GenerateNotes	= TRUE;
+static bool		DebugMode	= false;
+static bool		OptimizeMode	= false;
+static bool		GenerateLines	= true;
+static bool		GenerateNotes	= true;
 static int		TabStep		= GL_TAB_STEP_DEFAULT;
 static int		SysTabStep	= GL_TAB_STEP_SYSTEM;
 static int		LineWrapping	= GL_LINE_WRAP_DEFAULT;
 
-static BOOL		OptionIorOrequired = FALSE;
-static BOOL		GenerateH	= TRUE;
-static BOOL		GenerateCC	= TRUE;
+static bool		OptionIorOrequired = false;
+static bool		GenerateH	= true;
+static bool		GenerateCC	= true;
 
 
 	/*** the design ***/
@@ -287,9 +288,9 @@ static ERROR GetArguments(		/* parse command line arguments */
 	int	argc,
 	char	**argv)
 {
-GL_STRINGLIST	*Succ,
+gl_stringlist	*Succ,
 		*TmpDirList;
-static string	StdImportDir;
+static gl_string StdImportDir;
 const char	*TmpStr;
 ERROR		Error;
 int		Arg;
@@ -300,7 +301,7 @@ if (argc < 2)
 
 if (  (argc == 2)
     &&(0 == strcmp("-h", argv[1])))
-   { UsageInfoOnly = TRUE;
+   { UsageInfoOnly = true;
      return(SCC_ERROR_NO_ERROR);
     } /* fi */
 
@@ -326,14 +327,14 @@ FilenameOUT	= DesignName;
 
 assert(PDirList == NULL);		/* default import path */
 assert(PDirListInsert == NULL);
-PDirList = new GL_STRINGLIST();
+PDirList = new gl_stringlist();
 PDirList->Succ = NULL;
 PDirList->String = CCD_SPECC_STD_IMPORT1;
 if ((TmpStr = getenv(CCD_SPECC_ENV_VAR)))
    { StdImportDir = TmpStr;
      StdImportDir += "/";
      StdImportDir += CCD_SPECC_IMPORT_DIR;
-     PDirList->Succ = new GL_STRINGLIST();
+     PDirList->Succ = new gl_stringlist();
      PDirList->Succ->Succ = NULL;
      PDirList->Succ->String = StdImportDir.chars();
     } /* fi */
@@ -388,11 +389,11 @@ while(Arg < argc)			/* process all options */
 	  continue;
 	 } /* fi */
      if (0 == strcmp(&argv[Arg][1], "g"))
-	{ DebugMode = TRUE;
+	{ DebugMode = true;
 	  if (OptimizeMode)
 	     { GL_PrintWarning(GL_WARN_STANDARD,
 				"Option -g disables option -O");
-	       OptimizeMode = FALSE;
+	       OptimizeMode = false;
 	      } /* fi */
 	  if ((Error = CCD_SetCompilerMode(CCD_MODE_DEBUG)))
 	     { return(Error);
@@ -401,11 +402,11 @@ while(Arg < argc)			/* process all options */
 	  continue;
 	 } /* fi */
      if (0 == strcmp(&argv[Arg][1], "O"))
-	{ OptimizeMode = TRUE;
+	{ OptimizeMode = true;
 	  if (DebugMode)
 	     { GL_PrintWarning(GL_WARN_STANDARD,
 				"Option -O disables option -g");
-	       DebugMode = FALSE;
+	       DebugMode = false;
 	      } /* fi */
 	  if ((Error = CCD_SetCompilerMode(CCD_MODE_OPTIMIZE)))
 	     { return(Error);
@@ -414,12 +415,12 @@ while(Arg < argc)			/* process all options */
 	  continue;
 	 } /* fi */
      if (0 == strcmp(&argv[Arg][1], "sl"))
-	{ GenerateLines = FALSE;
+	{ GenerateLines = false;
 	  Arg++;
 	  continue;
 	 } /* fi */
      if (0 == strcmp(&argv[Arg][1], "sn"))
-	{ GenerateNotes = FALSE;
+	{ GenerateNotes = false;
 	  Arg++;
 	  continue;
 	 } /* fi */
@@ -488,7 +489,7 @@ while(Arg < argc)			/* process all options */
 	     { return(SCC_Error);
 	      } /* fi */
 	  if (OptionIorOrequired)
-	     { OptionIorOrequired = FALSE;
+	     { OptionIorOrequired = false;
 	      } /* fi */
 	  continue;
 	 } /* fi */
@@ -509,7 +510,7 @@ while(Arg < argc)			/* process all options */
 	     { return(SCC_Error);
 	      } /* fi */
 	  if (OptionIorOrequired)
-	     { OptionIorOrequired = FALSE;
+	     { OptionIorOrequired = false;
 	      } /* fi */
 	  continue;
 	 } /* fi */
@@ -528,12 +529,12 @@ while(Arg < argc)			/* process all options */
 	      } /* fi */
 	  else
 	     { if (DDefListLast)
-		  { DDefListLast->Succ = new GL_STRINGLIST();
+		  { DDefListLast->Succ = new gl_stringlist();
 		    DDefListLast = DDefListLast->Succ;
 		   } /* fi */
 	       else
 		  { assert(DDefList == NULL);
-		    DDefList = new GL_STRINGLIST();
+		    DDefList = new gl_stringlist();
 		    DDefListLast = DDefList;
 		   } /* esle */
 	       DDefListLast->Succ = NULL;
@@ -557,12 +558,12 @@ while(Arg < argc)			/* process all options */
 	      } /* fi */
 	  else
 	     { if (UDefListLast)
-		  { UDefListLast->Succ = new GL_STRINGLIST();
+		  { UDefListLast->Succ = new gl_stringlist();
 		    UDefListLast = UDefListLast->Succ;
 		   } /* fi */
 	       else
 		  { assert(UDefList == NULL);
-		    UDefList = new GL_STRINGLIST();
+		    UDefList = new gl_stringlist();
 		    UDefListLast = UDefList;
 		   } /* esle */
 	       UDefListLast->Succ = NULL;
@@ -586,12 +587,12 @@ while(Arg < argc)			/* process all options */
 	      } /* fi */
 	  else
 	     { if (IDirListLast)
-		  { IDirListLast->Succ = new GL_STRINGLIST();
+		  { IDirListLast->Succ = new gl_stringlist();
 		    IDirListLast = IDirListLast->Succ;
 		   } /* fi */
 	       else
 		  { assert(IDirList == NULL);
-		    IDirList = new GL_STRINGLIST();
+		    IDirList = new gl_stringlist();
 		    IDirListLast = IDirList;
 		   } /* esle */
 	       IDirListLast->Succ = NULL;
@@ -615,12 +616,12 @@ while(Arg < argc)			/* process all options */
 	      } /* fi */
 	  else
 	     { if (LDirListLast)
-		  { LDirListLast->Succ = new GL_STRINGLIST();
+		  { LDirListLast->Succ = new gl_stringlist();
 		    LDirListLast = LDirListLast->Succ;
 		   } /* fi */
 	       else
 		  { assert(LDirList == NULL);
-		    LDirList = new GL_STRINGLIST();
+		    LDirList = new gl_stringlist();
 		    LDirListLast = LDirList;
 		   } /* esle */
 	       LDirListLast->Succ = NULL;
@@ -644,12 +645,12 @@ while(Arg < argc)			/* process all options */
 	      } /* fi */
 	  else
 	     { if (lLibListLast)
-		  { lLibListLast->Succ = new GL_STRINGLIST();
+		  { lLibListLast->Succ = new gl_stringlist();
 		    lLibListLast = lLibListLast->Succ;
 		   } /* fi */
 	       else
 		  { assert(lLibList == NULL);
-		    lLibList = new GL_STRINGLIST();
+		    lLibList = new gl_stringlist();
 		    lLibListLast = lLibList;
 		   } /* esle */
 	       lLibListLast->Succ = NULL;
@@ -669,7 +670,7 @@ while(Arg < argc)			/* process all options */
 	       PDirListInsert = NULL;
 	      } /* fi */
 	  else
-	     { TmpDirList = new GL_STRINGLIST();
+	     { TmpDirList = new gl_stringlist();
 	       TmpDirList->Succ = NULL;
 	       TmpDirList->String = &argv[Arg][2];
 	       if (PDirList)
@@ -841,138 +842,137 @@ static ERROR GetCommand(		/* parse the command argument */
 {
 
 	/* ok, first we assume doing nothing */
-CallPreprocessor	= FALSE;
-CallParser		= FALSE;
-CallDeparser		= FALSE;
-CallTranslator		= FALSE;
-CallCompiler		= FALSE;
-CallLinker		= FALSE;
+CallPreprocessor	= false;
+CallParser		= false;
+CallDeparser		= false;
+CallTranslator		= false;
+CallCompiler		= false;
+CallLinker		= false;
 
 if (0 == strcmp(SuffixCmd, "sc2si"))	/* start from a .sc file */
-   { CallPreprocessor	= TRUE;
+   { CallPreprocessor	= true;
      InputFilename  = &FilenameSCin;
      OutputFilename = &FilenameSI;
      return(SCC_ERROR_NO_ERROR);
     } /* fi */
 if (0 == strcmp(SuffixCmd, "sc2cc"))
-   { CallPreprocessor	= TRUE;
-     CallParser		= TRUE;
-     CallTranslator	= TRUE;
+   { CallPreprocessor	= true;
+     CallParser		= true;
+     CallTranslator	= true;
      InputFilename  = &FilenameSCin;
      OutputFilename = &FilenameCC;
-     GenerateH		= FALSE;	/* no .h file */
+     GenerateH		= false;	/* no .h file */
      return(SCC_ERROR_NO_ERROR);
     } /* fi */
 if (0 == strcmp(SuffixCmd, "sc2h"))
-   { CallPreprocessor	= TRUE;
-     CallParser		= TRUE;
-     CallTranslator	= TRUE;
+   { CallPreprocessor	= true;
+     CallParser		= true;
+     CallTranslator	= true;
      InputFilename  = &FilenameSCin;
      OutputFilename = &FilenameH;
-     GenerateCC		= FALSE;	/* no .cc file */
+     GenerateCC		= false;	/* no .cc file */
      return(SCC_ERROR_NO_ERROR);
     } /* fi */
 if (0 == strcmp(SuffixCmd, "sc2cch"))
-   { CallPreprocessor	= TRUE;
-     CallParser		= TRUE;
-     CallTranslator	= TRUE;
+   { CallPreprocessor	= true;
+     CallParser		= true;
+     CallTranslator	= true;
      InputFilename  = &FilenameSCin;
      OutputFilename = NULL;	/* no -o option allowed */
      return(SCC_ERROR_NO_ERROR);
     } /* fi */
 if (0 == strcmp(SuffixCmd, "sc2o"))
-   { CallPreprocessor	= TRUE;
-     CallParser		= TRUE;
-     CallTranslator	= TRUE;
-     CallCompiler	= TRUE;
+   { CallPreprocessor	= true;
+     CallParser		= true;
+     CallTranslator	= true;
+     CallCompiler	= true;
      InputFilename  = &FilenameSCin;
      OutputFilename = &FilenameO;
      return(SCC_ERROR_NO_ERROR);
     } /* fi */
 if (0 == strcmp(SuffixCmd, "sc2out"))
-   { CallPreprocessor	= TRUE;
-     CallParser		= TRUE;
-     CallTranslator	= TRUE;
-     CallCompiler	= TRUE;
-     CallLinker		= TRUE;
+   { CallPreprocessor	= true;
+     CallParser		= true;
+     CallTranslator	= true;
+     CallCompiler	= true;
+     CallLinker		= true;
      InputFilename  = &FilenameSCin;
      OutputFilename = &FilenameOUT;
      return(SCC_ERROR_NO_ERROR);
     } /* fi */
 if (0 == strcmp(SuffixCmd, "sc2sc"))
-   { OptionIorOrequired = TRUE;
-     CallPreprocessor	= TRUE;
-     CallParser		= TRUE;
-     CallDeparser	= TRUE;
+   { OptionIorOrequired = true;
+     CallPreprocessor	= true;
+     CallParser		= true;
+     CallDeparser	= true;
      InputFilename  = &FilenameSCin;
      OutputFilename = &FilenameSCout;
      return(SCC_ERROR_NO_ERROR);
     } /* fi */
-
 if (0 == strcmp(SuffixCmd, "si2cc"))
-   { CallParser		= TRUE;
-     CallTranslator	= TRUE;
+   { CallParser		= true;
+     CallTranslator	= true;
      InputFilename  = &FilenameSI;
      OutputFilename = &FilenameCC;
-     GenerateH		= FALSE;	/* no .h file */
+     GenerateH		= false;	/* no .h file */
      return(SCC_ERROR_NO_ERROR);
     } /* fi */
 if (0 == strcmp(SuffixCmd, "si2h"))
-   { CallParser		= TRUE;
-     CallTranslator	= TRUE;
+   { CallParser		= true;
+     CallTranslator	= true;
      InputFilename  = &FilenameSI;
      OutputFilename = &FilenameH;
-     GenerateCC		= FALSE;	/* no .cc file */
+     GenerateCC		= false;	/* no .cc file */
      return(SCC_ERROR_NO_ERROR);
     } /* fi */
 if (0 == strcmp(SuffixCmd, "si2cch"))
-   { CallParser		= TRUE;
-     CallTranslator	= TRUE;
+   { CallParser		= true;
+     CallTranslator	= true;
      InputFilename  = &FilenameSI;
      OutputFilename = NULL;	/* no -o option allowed */
      return(SCC_ERROR_NO_ERROR);
     } /* fi */
 if (0 == strcmp(SuffixCmd, "si2o"))
-   { CallParser		= TRUE;
-     CallTranslator	= TRUE;
-     CallCompiler	= TRUE;
+   { CallParser		= true;
+     CallTranslator	= true;
+     CallCompiler	= true;
      InputFilename  = &FilenameSI;
      OutputFilename = &FilenameO;
      return(SCC_ERROR_NO_ERROR);
     } /* fi */
 if (0 == strcmp(SuffixCmd, "si2out"))
-   { CallParser		= TRUE;
-     CallTranslator	= TRUE;
-     CallCompiler	= TRUE;
-     CallLinker		= TRUE;
+   { CallParser		= true;
+     CallTranslator	= true;
+     CallCompiler	= true;
+     CallLinker		= true;
      InputFilename  = &FilenameSI;
      OutputFilename = &FilenameOUT;
      return(SCC_ERROR_NO_ERROR);
     } /* fi */
 if (0 == strcmp(SuffixCmd, "si2sc"))
-   { CallParser		= TRUE;
-     CallDeparser	= TRUE;
+   { CallParser		= true;
+     CallDeparser	= true;
      InputFilename  = &FilenameSI;
      OutputFilename = &FilenameSCout;
      return(SCC_ERROR_NO_ERROR);
     } /* fi */
 
 if (0 == strcmp(SuffixCmd, "cc2o"))	/* start from a .cc file */
-   { CallCompiler	= TRUE;
+   { CallCompiler	= true;
      InputFilename  = &FilenameCC;
      OutputFilename = &FilenameO;
      return(SCC_ERROR_NO_ERROR);
     } /* fi */
 if (0 == strcmp(SuffixCmd, "cc2out"))
-   { CallCompiler	= TRUE;
-     CallLinker		= TRUE;
+   { CallCompiler	= true;
+     CallLinker		= true;
      InputFilename  = &FilenameCC;
      OutputFilename = &FilenameOUT;
      return(SCC_ERROR_NO_ERROR);
     } /* fi */
 
 if (0 == strcmp(SuffixCmd, "o2out"))	/* start from a .o file */
-   { CallLinker		= TRUE;
+   { CallLinker		= true;
      InputFilename  = &FilenameO;
      OutputFilename = &FilenameOUT;
      return(SCC_ERROR_NO_ERROR);
@@ -990,7 +990,7 @@ return(SCC_ERROR_UNKNOWN_COMMAND);
 
 void CleanUp(void)
 {
-GL_STRINGLIST	*Succ;
+gl_stringlist	*Succ;
 
 while(PDirList)			/* clean up the import path */
    { Succ = PDirList->Succ;
@@ -1053,9 +1053,10 @@ int main(int argc, char **argv)
 int		i;
 ERROR		Error;
 FILE		*InFile;
-GL_HANDLE	*IO;
+gl_io		*IO;
+gl_string	Command;
 #ifdef SCC_DEBUG
-GL_STRINGLIST	*StringPtr;
+gl_stringlist	*StringPtr;
 #endif /* SCC_DEBUG */
 
 	/* initialization */
