@@ -4,10 +4,22 @@
 /* Author: Rainer Doemer			first version: 09/24/97 */
 /************************************************************************/
 
-/* last update: 11/29/01 */
+/* last update: 09/26/06 */
 
 /* modifications: (most recent first)
  *
+ * 09/26/06 PC  Adjustments for scrc 2.1
+ * 06/03/05 RD	reorganized and renamed global type names
+ * 12/17/04 RD	added support to output "Hankaku Kana" characters
+ *		in native encoding (not as octal escape sequence)
+ * 12/17/04 RD	disabled explicit support for EUC KANJI characters
+ *		as this is "automatically" supported (above 0x80)
+ * 12/17/04 RD	adjusted range for first SJIS KANJI character
+ *		from [\x81-\x9f\xe0-\xfc] to [\x81-\x9f\xe0-\xef]
+ * 06/15/04 PC  Adjustments for scrc 2.0
+ * 11/18/02 RD	improved SIR_Constant::GetMangledConstKey() method
+ * 11/09/02 RD	added SIR_Constant::GetMangledConstKey() methods
+ * 11/06/02 RD	added SIR_Constant::Compare() methods
  * 11/29/01 RD	provided isascii() macro for all platforms
  * 11/21/01 RD	took out default arguments from function definitions
  * 08/13/01 RD	switched type of bitvector length to unsigned
@@ -168,7 +180,7 @@ switch(Type)
 	  break;
 	 }
      default:
-	{ assert(FALSE);	/* bad Type */
+	{ assert(false);	/* bad Type */
 	 }
     } /* hctiws */
 
@@ -240,7 +252,7 @@ switch(Type)
 	  break;
 	 }
      default:
-	{ assert(FALSE);	/* bad Type */
+	{ assert(false);	/* bad Type */
 	 }
     } /* hctiws */
 
@@ -312,7 +324,7 @@ switch(Type)
 	  break;
 	 }
      default:
-	{ assert(FALSE);	/* bad Type */
+	{ assert(false);	/* bad Type */
 	 }
     } /* hctiws */
 
@@ -343,7 +355,7 @@ switch(Type)
 	  break;
 	 }
      default:
-	{ assert(FALSE);	/* bad Type */
+	{ assert(false);	/* bad Type */
 	 }
     } /* hctiws */
 
@@ -352,7 +364,7 @@ switch(Type)
 
 SIR_Constant::SIR_Constant(		/* constructor #5 */
 	SIR_CONSTTYPE	Type,
-	string		*Value,	/* (the string is consumed!) */
+	gl_string	*Value,	/* (the string is consumed!) */
 	unsigned int	Line /* = 0 */,
 	sir_fileinfo	*FileInfo /* = NULL */) :
 		SIR_Node(Line, FileInfo)
@@ -366,7 +378,7 @@ switch(Type)
 	  break;
 	 }
      default:
-	{ assert(FALSE);	/* bad Type */
+	{ assert(false);	/* bad Type */
 	 }
     } /* hctiws */
 
@@ -393,11 +405,11 @@ switch(Type)
 	  break;
 	 }
      case SIR_CONST_CHARSTRING:
-	{ CS_Value = new string(Value);
+	{ CS_Value = new gl_string(Value);
 	  break;
 	 }
      default:
-	{ assert(FALSE);	/* bad Type */
+	{ assert(false);	/* bad Type */
 	 }
     } /* hctiws */
 
@@ -488,11 +500,11 @@ switch(Original->Type)
 	  break;
 	 }
      case SIR_CONST_CHARSTRING:
-	{ CS_Value = new string(*Original->CS_Value);
+	{ CS_Value = new gl_string(*Original->CS_Value);
 	  break;
 	 }
      default:
-	{ assert(FALSE);	/* bad Type */
+	{ assert(false);	/* bad Type */
 	 }
     } /* hctiws */
 
@@ -580,7 +592,7 @@ switch(Type)
 	{ break;			/* nothing to promote */
 	 }
      default:
-	{ assert(FALSE);	/* bad Type */
+	{ assert(false);	/* bad Type */
 	 }
     } /* hctiws */
 
@@ -673,10 +685,10 @@ if (  (Type == SIR_CONST_BIT)	/* first handle conversions to bitvectors */
 	       break;
 	      }
 	  case SIR_CONST_CHARSTRING:
-	     { assert(FALSE);	/* not convertable to bitvector */
+	     { assert(false);	/* not convertable to bitvector */
 	      }
 	  default:
-	     { assert(FALSE);	/* bad constant type */
+	     { assert(false);	/* bad constant type */
 	      }
 	 } /* hctiws */
      assert((Type == SIR_CONST_UBIT) == (BIT_Value->isUnsigned()));
@@ -759,7 +771,7 @@ switch(this->Type)
 	  break;
 	 }
      default:
-	{ assert(FALSE);	/* bad Type */
+	{ assert(false);	/* bad Type */
 	 }
     } /* hctiws */
 
@@ -833,7 +845,7 @@ return(&Pretty[0]);
 
 
 void SIR_Constant::PrettyString(	/* returns a C-style string */
-	string		*Result,
+	gl_string	*Result,
 	const char	*String)
 {
 const char	*CharPtr;
@@ -852,7 +864,7 @@ while(*CharPtr)
      c1 = *CharPtr;
      c2 = *(CharPtr+1);
      if (  (  (c1 >= 0x81 && c1 <= 0x9f)
-	    ||(c1 >= 0xe0 && c1 <= 0xfc))
+	    ||(c1 >= 0xe0 && c1 <= 0xef))
 	 &&(  (c2 >= 0x40 && c2 <= 0xfc && c2 != 0x7f)))
 	{ /* this looks like a (2-byte) S-JIS code */
 	  *Result += c1;
@@ -860,14 +872,20 @@ while(*CharPtr)
 	  CharPtr += 2;
 	  continue;
 	 } /* fi */
-     if (  (c1 >= 0xa1 && c1 <= 0xfe)
-	 &&(c2 >= 0xa1 && c2 <= 0xfe))
-	{ /* this looks like a (2-byte) EUC code */
+     if (c1 >= 0xa1 && c1 <= 0xdf)
+	{ /* this looks like a (1-byte) "Hankaku Kana" code */
 	  *Result += c1;
-	  *Result += c2;
-	  CharPtr += 2;
+	  CharPtr++;
 	  continue;
 	 } /* fi */
+//   if (  (c1 >= 0xa1 && c1 <= 0xfe)
+//	 &&(c2 >= 0xa1 && c2 <= 0xfe))
+//	{ /* this looks like a (2-byte) EUC code */
+//	  *Result += c1;
+//	  *Result += c2;
+//	  CharPtr += 2;
+//	  continue;
+//	 } /* fi */
 #endif /* SUPPORT_KANJI */
 
      if (*CharPtr == '\'')
@@ -887,11 +905,67 @@ while(*CharPtr)
 
 } /* end of SIR_Constant::PrettyString */
 
+/* PC No Error Check*/
+sir_constant *SIR_Constant::New(        /* create a new integer constant (#1) */
+        LONG_LONG       Value,          /* (returns NULL if SIR_Error) */
+        SIR_CONSTTYPE   CType /* = SIR_CONST_INT */)    /* default: integer */
+{
+sir_constant    *NewConst;
+
+switch(CType)
+   { case SIR_CONST_BOOL:
+     case SIR_CONST_CHAR:
+     case SIR_CONST_UCHAR:
+     case SIR_CONST_SHORT:
+     case SIR_CONST_USHORT:
+     case SIR_CONST_INT:
+     case SIR_CONST_UINT:
+     case SIR_CONST_LONG:
+     case SIR_CONST_ULONG:
+     case SIR_CONST_LONGLONG:
+     case SIR_CONST_ULONGLONG:
+     case SIR_CONST_BIT:
+        { NewConst = new SIR_Constant(CType, Value);
+          break;
+         }
+     case SIR_CONST_FLOAT:
+     case SIR_CONST_DOUBLE:
+     case SIR_CONST_LONGDOUBLE:
+     case SIR_CONST_UBIT:
+     case SIR_CONST_CHARSTRING:
+     default:
+        { 
+          return(NULL);
+         }
+    } /* hctiws */
+
+return(NewConst);
+
+} /* end of SIR_Constant::New (#1) */
+
+sir_constant *SIR_Constant::Copy(       /* creates a copy of the constant */
+        bool            Strip /* = true */) /* (returns NULL if SIR_Error) */
+{
+sir_constant    *Const;
+
+Const = new SIR_Constant(this);
+
+return(Const);
+
+} /* end of SIR_Constant::Copy */
+
+
+void SIR_Constant::Delete(void)         /* deletes a constant */
+{
+
+delete this;
+
+} /* end of SIR_Constant::Delete */
 
 const char *SIR_Constant::Print(	/* return text string of value */
-	BOOL		CppNotation /* = FALSE */) /* default: SpecC notation */
+	bool		CppNotation /* = false */) /* default: SpecC notation */
 {
-static string	PrintedValue;
+static gl_string	PrintedValue;
 char		NumberBuffer[MAX_NUM_LENGTH];
 char		*BitBuffer,
 		*TmpValue;
@@ -1131,7 +1205,7 @@ switch(Type)
 	  break;
 	 }
      default:
-	{ assert(FALSE);	/* bad Type */
+	{ assert(false);	/* bad Type */
 	 }
     } /* hctiws */
 
@@ -1140,6 +1214,195 @@ return(PrintedValue.chars());
 } /* end of SIR_Constant::Print */
 
 
+ERROR SIR_Constant::Compare(    /* compare this constant with another one (#1)*/
+        sir_constant    *Const2,        /* (returns error if not comparable) */
+        int             *Result)        /* Result is {-1,0,1} as in strcmp() */
+{
+sir_constant    *C1,
+                *C2;
+
+assert(Const2 != NULL);
+assert(Result != NULL);
+
+        /* first, handle special case of string comparisons */
+
+if (  (Type == SIR_CONST_CHARSTRING)
+    ||(Const2->Type == SIR_CONST_CHARSTRING))
+   { 
+     /* perform a STRING-type comparison */
+     *Result = strcmp(CS_Value->chars(), Const2->CS_Value->chars());
+     return(SIR_ERROR_NO_ERROR);
+    } /* fi */
+
+        /* second, handle floating point comparisons */
+
+if (  (Type == SIR_CONST_FLOAT)
+    ||(Type == SIR_CONST_DOUBLE)
+    ||(Type == SIR_CONST_LONGDOUBLE)
+    ||(Const2->Type == SIR_CONST_FLOAT)
+    ||(Const2->Type == SIR_CONST_DOUBLE)
+    ||(Const2->Type == SIR_CONST_LONGDOUBLE))
+   { /* perform a LD-type comparison */
+     C1 = new SIR_Constant(this);
+     if (C1->Type != SIR_CONST_LONGDOUBLE)
+        { C1->Converted(SIR_CONST_LONGDOUBLE);
+         } /* fi */
+     C2 = new SIR_Constant(Const2);
+     if (C2->Type != SIR_CONST_LONGDOUBLE)
+        { C2->Converted(SIR_CONST_LONGDOUBLE);
+         } /* fi */
+     if (C1->LD_Value > C2->LD_Value)
+        { *Result = 1;
+         } /* fi */
+     else
+        { if (C1->LD_Value < C2->LD_Value)
+             { *Result = -1;
+              } /* fi */
+          else
+             { *Result = 0;
+              } /* esle */
+         } /* esle */
+     delete C1;
+     delete C2;
+     return(SIR_ERROR_NO_ERROR);
+    } /* fi */
+
+        /* third, handle bit vector comparisons */
+
+if (  (Type == SIR_CONST_BIT)
+    ||(Type == SIR_CONST_UBIT)
+    ||(Const2->Type == SIR_CONST_BIT)
+    ||(Const2->Type == SIR_CONST_UBIT))
+   { /* perform a BIT-type comparison */
+     C1 = new SIR_Constant(this);
+     if (  (C1->Type == SIR_CONST_CHAR)
+         ||(C1->Type == SIR_CONST_SHORT)
+         ||(C1->Type == SIR_CONST_INT)
+         ||(C1->Type == SIR_CONST_LONG)
+         ||(C1->Type == SIR_CONST_LONGLONG))
+        { C1->Converted(SIR_CONST_BIT);
+         } /* fi */
+     else
+        { if (  (C1->Type == SIR_CONST_BOOL)
+              ||(C1->Type == SIR_CONST_UCHAR)
+              ||(C1->Type == SIR_CONST_USHORT)
+              ||(C1->Type == SIR_CONST_UINT)
+              ||(C1->Type == SIR_CONST_ULONG)
+              ||(C1->Type == SIR_CONST_ULONGLONG))
+             { C1->Converted(SIR_CONST_UBIT);
+              } /* fi */
+         } /* esle */
+     C2 = new SIR_Constant(Const2);
+     if (  (C2->Type == SIR_CONST_CHAR)
+         ||(C2->Type == SIR_CONST_SHORT)
+         ||(C2->Type == SIR_CONST_INT)
+         ||(C2->Type == SIR_CONST_LONG)
+         ||(C2->Type == SIR_CONST_LONGLONG))
+        { C2->Converted(SIR_CONST_BIT);
+         } /* fi */
+     else
+        { if (  (C2->Type == SIR_CONST_BOOL)
+              ||(C2->Type == SIR_CONST_UCHAR)
+              ||(C2->Type == SIR_CONST_USHORT)
+              ||(C2->Type == SIR_CONST_UINT)
+              ||(C2->Type == SIR_CONST_ULONG)
+              ||(C2->Type == SIR_CONST_ULONGLONG))
+             { C2->Converted(SIR_CONST_UBIT);
+              } /* fi */
+         } /* esle */
+     if (*C1->BIT_Value > *C2->BIT_Value)
+        { *Result = 1;
+         } /* fi */
+     else
+        { if (*C1->BIT_Value < *C2->BIT_Value)
+             { *Result = -1;
+              } /* fi */
+          else
+             { *Result = 0;
+              } /* esle */
+         } /* esle */
+     delete C1;
+     delete C2;
+     return(SIR_ERROR_NO_ERROR);
+    } /* fi */
+
+        /* forth, handle unsigned long long comparisons */
+
+if (  (Type == SIR_CONST_ULONGLONG)
+    ||(Const2->Type == SIR_CONST_ULONGLONG))
+   { /* perform a ULL-type comparison */
+     C1 = new SIR_Constant(this);
+     if (C1->Type != SIR_CONST_ULONGLONG)
+        { C1->Converted(SIR_CONST_ULONGLONG);
+         } /* fi */
+     C2 = new SIR_Constant(Const2);
+     if (C2->Type != SIR_CONST_ULONGLONG)
+        { C2->Converted(SIR_CONST_ULONGLONG);
+         } /* fi */
+     if (C1->ULL_Value > C2->ULL_Value)
+        { *Result = 1;
+         } /* fi */
+     else
+        { if (C1->ULL_Value < C2->ULL_Value)
+             { *Result = -1;
+              } /* fi */
+          else
+             { *Result = 0;
+              } /* esle */
+         } /* esle */
+     delete C1;
+     delete C2;
+     return(SIR_ERROR_NO_ERROR);
+    } /* fi */
+
+        /* finally, handle the other integral comparisons */
+
+/* perform a LL-type comparison */
+C1 = new SIR_Constant(this);
+if (C1->Type != SIR_CONST_LONGLONG)
+   { C1->Converted(SIR_CONST_LONGLONG);
+    } /* fi */
+C2 = new SIR_Constant(Const2);
+if (C2->Type != SIR_CONST_LONGLONG)
+   { C2->Converted(SIR_CONST_LONGLONG);
+    } /* fi */
+if (C1->LL_Value > C2->LL_Value)
+   { *Result = 1;
+    } /* fi */
+else
+   { if (C1->LL_Value < C2->LL_Value)
+        { *Result = -1;
+         } /* fi */
+     else
+        { *Result = 0;
+         } /* esle */
+    } /* esle */
+delete C1;
+delete C2;
+
+return(SIR_ERROR_NO_ERROR);
+
+} /* end of SIR_Constant::Compare #1 */
+
+
+int SIR_Constant::Compare(      /* compare two comparable constants (#2) */
+        sir_constant    *Const1,        /* result is {-1,0,1} as in strcmp() */
+        sir_constant    *Const2)
+{
+ERROR           Error;
+int             Result;
+
+assert(Const1 != NULL);
+assert(Const2 != NULL);
+
+Error = Const1->Compare(Const2, &Result);
+assert(Error == SIR_ERROR_NO_ERROR);
+
+return(Result);
+
+} /* end of SIR_Constant::Compare #2 */
+
+
 /*** internal functions *************************************************/
 
 
@@ -1208,13 +1471,13 @@ switch(Type)
 	 }
      case SIR_CONST_BIT:
      case SIR_CONST_UBIT:
-	{ assert(FALSE);	/* is handled elsewhere */
+	{ assert(false);	/* is handled elsewhere */
 	 }
      case SIR_CONST_CHARSTRING:
-	{ assert(FALSE);	/* cannot be converted */
+	{ assert(false);	/* cannot be converted */
 	 }
      default:
-	{ assert(FALSE);	/* bad Type */
+	{ assert(false);	/* bad Type */
 	 }
     } /* hctiws */
 
@@ -1288,13 +1551,13 @@ switch(Type)
 	 }
      case SIR_CONST_BIT:
      case SIR_CONST_UBIT:
-	{ assert(FALSE);	/* is handled elsewhere */
+	{ assert(false);	/* is handled elsewhere */
 	 }
      case SIR_CONST_CHARSTRING:
-	{ assert(FALSE);	/* cannot be converted */
+	{ assert(false);	/* cannot be converted */
 	 }
      default:
-	{ assert(FALSE);	/* bad Type */
+	{ assert(false);	/* bad Type */
 	 }
     } /* hctiws */
 
@@ -1370,13 +1633,13 @@ switch(Type)
 	 }
      case SIR_CONST_BIT:
      case SIR_CONST_UBIT:
-	{ assert(FALSE);	/* is handled elsewhere */
+	{ assert(false);	/* is handled elsewhere */
 	 }
      case SIR_CONST_CHARSTRING:
-	{ assert(FALSE);	/* cannot be converted */
+	{ assert(false);	/* cannot be converted */
 	 }
      default:
-	{ assert(FALSE);	/* bad Type */
+	{ assert(false);	/* bad Type */
 	 }
     } /* hctiws */
 
@@ -1462,13 +1725,13 @@ switch(Type)
 	 }
      case SIR_CONST_BIT:
      case SIR_CONST_UBIT:
-	{ assert(FALSE);	/* handled earlier */
+	{ assert(false);	/* handled earlier */
 	 }
      case SIR_CONST_CHARSTRING:
-	{ assert(FALSE);	/* cannot be converted */
+	{ assert(false);	/* cannot be converted */
 	 }
      default:
-	{ assert(FALSE);	/* bad Type */
+	{ assert(false);	/* bad Type */
 	 }
     } /* hctiws */
 
