@@ -4,10 +4,29 @@
 /* Author: Rainer Doemer			first version: 09/24/97 */
 /************************************************************************/
 
-/* last update: 05/31/01 */
+/* last update: 09/26/06 */
 
 /* modifications: (most recent first)
  *
+ * 09/26/06 PC  Adjustments for scrc 2.1
+ * 06/03/05 RD	reorganized and renamed global type names
+ * 06/23/04 PC  Fixed a bug in the earlier EOL related changes.
+ * 06/21/04 PC  Introduced Macro EOL
+ * 06/15/04 PC  Adjustments for scrc 2.0
+ * 02/13/03 RD	added support for notes at fsmd states
+ * 09/19/02 RD	added SetNewSource parameter to SpecC code generation
+ * 06/27/02 RD	converted log to a log list utilizing composite annotations
+ * 05/20/02 RD	removed default arguments from some function definitions
+ * 05/16/02 RD	adjusted new annotation methods for composite contents;
+ *		removed extra newline after global annotations
+ * 02/15/02 RD	bug fix: user-type declarations that are later used 'const'
+ *		made 'import' fail with error #2037 (see HeaderFile.sc)
+ * 01/30/02 RD	allow indexed access to elements of composite annotations
+ * 01/15/02 RD	added support for composite annotations
+ * 11/21/01 RD	took out default arguments from function definitions
+ * 11/09/01 RD	added line wrapping in code generation
+ * 11/08/01 RD	switched code generation to use GL_IO layer
+ * 10/04/01 RD	improved indentation of generated code
  * 05/31/01 RD	eliminated level 2 of SIR API
  * 05/26/01 RD	bug fix for source code 'import'
  * 05/25/01 RD	eliminated support for binary SIR files (import/export)
@@ -19,13 +38,18 @@
  * 04/06/01 RD	started this header (last update was 04/08/99)
  */
 
+#ifdef LINUX
+#define _XOPEN_SOURCE /* glibc2 needs this */
+#endif /* LINUX */
+
 #include "IntRep/Design.h"
 #include "IntRep/Extern.h"
 
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
-
+#include <ctype.h>
+#include <errno.h>
 
 /*** constants and macros ***********************************************/
 
@@ -49,21 +73,18 @@
 /*** prototypes of internal functions ***********************************/
 
 
-static BOOL NeedForRangeChecker(	/* checks if range checker is needed */
+static bool NeedForRangeChecker(	/* checks if range checker is needed */
 	sir_design	*Design);
 
 static ERROR DeclareRangeChecker(	/* generate decl. of range checker */
-	FILE		*File);
+	gl_io		*IO);
 
 static ERROR DefineRangeChecker(	/* generate def. of range checker */
-	FILE		*File);
-
+	gl_io		*IO);
 
 /*** exported variables *************************************************/
 
 
-	/* none */
-
 
 /*** internal variables *************************************************/
 
@@ -268,7 +289,7 @@ return(SIR_ERROR_NO_ERROR);
 } /* end of SIR_Design::DFS_ForAllStatements */
 
 
-ERROR SIR_Design::Annotate(	/* attach a (global) note to the design */
+ERROR SIR_Design::Annotate(	/* attach a (global) note to the design (#1) */
 	sir_note	*NewNote)	/* (consumes NewNote) */
 {
 
@@ -277,6 +298,7 @@ assert(NewNote != NULL);
 NewNote->Symbol = NULL;		/* global Note (no back pointers) */
 NewNote->UserType = NULL;
 NewNote->Label = NULL;
+NewNote->State = NULL;
 
 if (Notes)
    { if (! Notes->Insert(NewNote))
@@ -289,15 +311,14 @@ else
     } /* esle */
 
 return(SIR_ERROR_NO_ERROR);
-
-} /* end of SIR_Design::Annotate */
+} /* end of SIR_Design::Annotate (#1) */
 
 
 ERROR SIR_Design::Integrate(		/* integrates an imported design */
 	sir_design	*Imported,	/* (consumes the imported design) */
-	const char	*ImportFile = NULL,	/* (default: no warnings) */
-	const char	*SourceFile = NULL,
-	unsigned int	LineNumber = 0)
+	const char	*ImportFile /* = NULL */, /* (default: no warnings) */
+	const char	*SourceFile /* = NULL */,
+	unsigned int	LineNumber /* = 0 */)
 {
 sir_import	*NewImport,
 		*DoubleImport;
@@ -482,7 +503,9 @@ if ((SIR_Error = Symbols->UserTypes->Integrate(	/* merge top-level UserTypes */
    { delete Imported;
      return(SIR_Error);
     } /* fi */
-/* we postpone pointer-unaliasing after type integration */
+Imported->Types->UnAlias();	// bug fix: (02/15/02, RD)
+				// see SIR_UserTypes::Integrate()!
+/* we postpone any other pointer-unaliasing until after type integration */
 
 	/* step 5: integrate types */
 
@@ -544,150 +567,177 @@ Types->GarbageCollection();		/* clean up type table */
 
 } /* end of SIR_Design::GarbageCollection */
 
-
+//PC 06/21/04
 ERROR SIR_Design::WriteSC(	/* (re-) generates SpecC source code */
-	FILE		*File,
+	gl_io		*IO,
 	const char	*Filename,	/* (full path) */
-	BOOL		WriteNotes = TRUE,
-	BOOL		WriteLines = TRUE)
+	bool		WriteNotes /* = true */,
+	bool		WriteLines /* = true */)
 {
 time_t		SysTime;
-unsigned int	CurrLineNo;
+unsigned int	i;
+gl_string	TmpString;
 
-assert(File != NULL);
+assert(IO != NULL);
 assert(Filename != NULL);
 
 time(&SysTime);		/* read current time and date */
-CurrLineNo = 1;
 
-fprintf(File,
-	"///////////////////////////////////////////////////////////"
-								"///////////\n"
-	"// SpecC source code generated by " GL_SPECC_NAME " V"
-						GL_SPECC_VERSION_STRING "\n"
-	"// Design: %s\n"
-	"// File:   %s\n"
-	"// Time:   %s"
-	"///////////////////////////////////////////////////////////"
-								"///////////\n",
-	Name.chars(),
-	GL_GetFileFromPath(Filename),
-	ctime(&SysTime));
-CurrLineNo += 6;
+IO->PutS("//");
+for(i=IO->GetColNo(); i<IO->GetLineWrap(); i++)
+   { IO->PutC('/');
+    } /* rof */
+IO->PutNL();
+IO->PutS("// SpecC source code generated by " GL_SPECC_NAME " V"
+						GL_SPECC_VERSION_STRING EOL);
+IO->PutS("// Design: ");
+IO->PutS(Name);
+IO->PutNL();
+IO->PutS("// File:   ");
+IO->PutS(GL_GetFileFromPath(Filename));
+IO->PutNL();
+IO->PutS("// Time:   ");
 
-if (ferror(File))
-   { SIR_ErrMsg.form("Writing to file failed%s",
-			GL_SystemErrorMessage());
-     return(SIR_ERROR_WRITING_SPECC_FILE);
+/*Replace the end of line character with one choosen by user*/
+TmpString = ctime(&SysTime);
+assert(TmpString[TmpString.length()-1] == '\n');
+TmpString = TmpString.before((int)(TmpString.length()-1));
+IO->PutS(TmpString+EOL);
+
+IO->PutS("//");
+for(i=IO->GetColNo(); i<IO->GetLineWrap(); i++)
+   { IO->PutC('/');
+    } /* rof */
+IO->PutNL();
+
+if ((SIR_Error = IO->Check()))
+   { return(SIR_Error);
     } /* fi */
 
+
 if (! WriteNotes)
-   { fputs("\n// Note: Notes are suppressed in this file.\n", File);
-     CurrLineNo += 2;
+   { IO->PutNL();
+     IO->PutS("// Note: Notes are suppressed in this file." EOL);
     } /* fi */
 if (! WriteLines)
-   { fputs("\n// Note: Line infos are suppressed in this file.\n", File);
-     CurrLineNo += 2;
+   { IO->PutNL();
+     IO->PutS("// Note: Line infos are suppressed in this file." EOL);
     } /* fi */
 
-SIR_LineInfo::InitWriteSC(WriteLines,	/* init for new file */
-			Filename, CurrLineNo);
+SIR_LineInfo::InitWriteSC(WriteLines, Filename);
 
 if (Notes)			/* write all global notes */
-   { if ((SIR_Error = Notes->WriteSC(File, WriteNotes)))
+   { if ((SIR_Error = Notes->WriteSC(IO, WriteNotes)))
 	{ return(SIR_Error);
 	 } /* fi */
     } /* fi */
 
 if (Symbols)			/* write all global decls. and defs. */
-   { if ((SIR_Error = Symbols->WriteSC(File, WriteNotes)))
+   { if ((SIR_Error = Symbols->WriteSC(IO, WriteNotes)))
 	{ return(SIR_Error);
 	 } /* fi */
     } /* fi */
 
-fprintf(File,
-	"\n\n"
-	"///////////////////////////////////////////////////////////"
-								"///////////\n"
-	"// End of file %s\n"
-	"///////////////////////////////////////////////////////////"
-								"///////////\n",
-	GL_GetFileFromPath(Filename));
+if (IO->GetColNo() > 0)
+   { IO->PutNL();
+    } /* fi */
+IO->PutNL();
+IO->PutS("//");
+for(i=IO->GetColNo(); i<IO->GetLineWrap(); i++)
+   { IO->PutC('/');
+    } /* rof */
+IO->PutNL();
+IO->PutS("// End of file ");
+IO->PutS(GL_GetFileFromPath(Filename));
+IO->PutNL();
+IO->PutS("//");
+for(i=IO->GetColNo(); i<IO->GetLineWrap(); i++)
+   { IO->PutC('/');
+    } /* rof */
+IO->PutNL();
 
-if (ferror(File))
-   { SIR_ErrMsg.form("Writing to file failed%s",
-			GL_SystemErrorMessage());
-     return(SIR_ERROR_WRITING_SPECC_FILE);
+if ((SIR_Error = IO->Check()))
+   { return(SIR_Error);
     } /* fi */
 
 return(SIR_ERROR_NO_ERROR);	/* successful */
 
 } /* end of SIR_Design::WriteSC */
 
-
+//PC 06/21/04
 ERROR SIR_Design::WriteH(	/* generates C++ source code: header file */
-	FILE		*File,
+	gl_io		*IO,
 	const char	*Filename,	/* (full path) */
-	BOOL		WriteNotes = TRUE,
-	BOOL		WriteLines = TRUE)
+	bool		WriteNotes /* = true */,
+	bool		WriteLines /* = true */)
 {
 time_t		SysTime;
-unsigned int	CurrLineNo;
+unsigned int	i;
+gl_string	TmpString;
 
-assert(File != NULL);
+assert(IO != NULL);
 assert(Filename != NULL);
 
 time(&SysTime);		/* read current time and date */
-CurrLineNo = 1;
 
-fprintf(File,
-	"///////////////////////////////////////////////////////////"
-								"///////////\n"
-	"// C++ header file generated by " GL_SPECC_NAME " V"
-						GL_SPECC_VERSION_STRING "\n"
-	"// Design: %s\n"
-	"// File:   %s\n"
-	"// Time:   %s"
-	"///////////////////////////////////////////////////////////"
-								"///////////\n",
-	Name.chars(),
-	GL_GetFileFromPath(Filename),
-	ctime(&SysTime));
-CurrLineNo += 6;
+IO->PutS("//");
+for(i=IO->GetColNo(); i<IO->GetLineWrap(); i++)
+   { IO->PutC('/');
+    } /* rof */
+IO->PutNL();
+IO->PutS("// C++ header file generated by " GL_SPECC_NAME " V"
+						GL_SPECC_VERSION_STRING EOL);
+IO->PutS("// Design: ");
+IO->PutS(Name);
+IO->PutNL();
+IO->PutS("// File:   ");
+IO->PutS(GL_GetFileFromPath(Filename));
+IO->PutNL();
+IO->PutS("// Time:   ");
 
-if (ferror(File))
-   { SIR_ErrMsg.form("Writing to file failed%s",
-			GL_SystemErrorMessage());
-     return(SIR_ERROR_WRITING_SPECC_FILE);
+/*Replace the end of line character with one choosen by user*/
+TmpString = ctime(&SysTime);
+assert(TmpString[TmpString.length()-1] == '\n');
+TmpString = TmpString.before((int)(TmpString.length()-1));
+IO->PutS(TmpString+EOL);
+
+IO->PutS("//");
+for(i=IO->GetColNo(); i<IO->GetLineWrap(); i++)
+   { IO->PutC('/');
+    } /* rof */
+IO->PutNL();
+
+if ((SIR_Error = IO->Check()))
+   { return(SIR_Error);
     } /* fi */
 
+
 if (! WriteNotes)
-   { fputs("\n// Note: Notes are suppressed in this file.\n", File);
-     CurrLineNo += 2;
+   { IO->PutNL();
+     IO->PutS("// Note: Notes are suppressed in this file." EOL);
     } /* fi */
 if (! WriteLines)
-   { fputs("\n// Note: Line infos are suppressed in this file.\n", File);
-     CurrLineNo += 2;
+   { IO->PutNL();
+     IO->PutS("// Note: Line infos are suppressed in this file." EOL);
     } /* fi */
 
-fputs("\n#include <" SIR_SIM_HEADER_NAME ">", File);
-CurrLineNo += 1;
-fputs("\n#include <" SIR_LLONG_HEADER_NAME ">\n", File);
-CurrLineNo += 2;
-fputs("\n#include <" SIR_BIT_HEADER_NAME ">\n", File);
-CurrLineNo += 2;
+IO->PutNL();
+IO->PutS("#include <" SIR_SIM_HEADER_NAME ">");
+IO->PutNL();
+IO->PutS("#include <" SIR_LLONG_HEADER_NAME ">");
+IO->PutNL();
+IO->PutS("#include <" SIR_BIT_HEADER_NAME ">" EOL);
 
-SIR_LineInfo::InitWriteSC(WriteLines,	/* init for new file */
-			Filename, CurrLineNo);
+SIR_LineInfo::InitWriteSC(WriteLines, Filename);
 
 if (Notes)			/* write all global notes */
-   { if ((SIR_Error = Notes->WriteSC(File, WriteNotes, TRUE)))
+   { if ((SIR_Error = Notes->WriteSC(IO, WriteNotes, true)))
 	{ return(SIR_Error);
 	 } /* fi */
     } /* fi */
 
 if (Symbols)			/* translate all global declarations */
-   { if ((SIR_Error = Symbols->WriteH(File, WriteNotes)))
+   { if ((SIR_Error = Symbols->WriteH(IO, WriteNotes)))
 	{ return(SIR_Error);
 	 } /* fi */
     } /* fi */
@@ -700,111 +750,135 @@ if (NeedForRangeChecker(this))
 		"global symbol '" SIR_CXX_RANGE_NAME "' already exists");
 	 } /* fi */
      else
-	{ if ((SIR_Error = DeclareRangeChecker(File)))
+	{ if ((SIR_Error = DeclareRangeChecker(IO)))
 	     { return(SIR_Error);
 	      } /* fi */
 	 } /* esle */
     } /* fi */
 
-fprintf(File,
-	"\n\n"
-	"///////////////////////////////////////////////////////////"
-								"///////////\n"
-	"// End of file %s\n"
-	"///////////////////////////////////////////////////////////"
-								"///////////\n",
-	GL_GetFileFromPath(Filename));
+if (IO->GetColNo() > 0)
+   { IO->PutNL();
+    } /* fi */
+IO->PutNL();
+IO->PutS("//");
+for(i=IO->GetColNo(); i<IO->GetLineWrap(); i++)
+   { IO->PutC('/');
+    } /* rof */
+IO->PutNL();
+IO->PutS("// End of file ");
+IO->PutS(GL_GetFileFromPath(Filename));
+IO->PutNL();
+IO->PutS("//");
+for(i=IO->GetColNo(); i<IO->GetLineWrap(); i++)
+   { IO->PutC('/');
+    } /* rof */
+IO->PutNL();
 
-if (ferror(File))
-   { SIR_ErrMsg.form("Writing to file failed%s",
-			GL_SystemErrorMessage());
-     return(SIR_ERROR_WRITING_SPECC_FILE);
+if ((SIR_Error = IO->Check()))
+   { return(SIR_Error);
     } /* fi */
 
 return(SIR_ERROR_NO_ERROR);	/* successful */
 
 } /* end of SIR_Design::WriteH */
 
-
+//PC 06/21/04
 ERROR SIR_Design::WriteCC(	/* generates C++ source code: main file */
-	FILE		*File,
+	gl_io		*IO,
 	const char	*Filename,	/* (full path) */
 	const char	*Headername,	/* (full path) */
-	BOOL		WriteNotes = TRUE,
-	BOOL		WriteLines = TRUE)
+	bool		WriteNotes /* = true */,
+	bool		WriteLines /* = true */)
 {
 time_t		SysTime;
-unsigned int	CurrLineNo;
+unsigned int	i;
+gl_string	TmpString;
 
-assert(File != NULL);
+assert(IO != NULL);
 assert(Filename != NULL);
 assert(Headername != NULL);
 
 time(&SysTime);		/* read current time and date */
-CurrLineNo = 1;
 
-fprintf(File,
-	"///////////////////////////////////////////////////////////"
-								"///////////\n"
-	"// C++ source file generated by " GL_SPECC_NAME " V"
-						GL_SPECC_VERSION_STRING "\n"
-	"// Design: %s\n"
-	"// File:   %s\n"
-	"// Time:   %s"
-	"///////////////////////////////////////////////////////////"
-								"///////////\n",
-	Name.chars(),
-	GL_GetFileFromPath(Filename),
-	ctime(&SysTime));
-CurrLineNo += 6;
+IO->PutS("//");
+for(i=IO->GetColNo(); i<IO->GetLineWrap(); i++)
+   { IO->PutC('/');
+    } /* rof */
+IO->PutNL();
+IO->PutS("// C++ source file generated by " GL_SPECC_NAME " V"
+						GL_SPECC_VERSION_STRING EOL);
+IO->PutS("// Design: ");
+IO->PutS(Name);
+IO->PutNL();
+IO->PutS("// File:   ");
+IO->PutS(GL_GetFileFromPath(Filename));
+IO->PutNL();
+IO->PutS("// Time:   ");
 
-if (ferror(File))
-   { SIR_ErrMsg.form("Writing to file failed%s",
-			GL_SystemErrorMessage());
-     return(SIR_ERROR_WRITING_SPECC_FILE);
+/*Replace the end of line character with one choosen by user*/
+TmpString = ctime(&SysTime);
+assert(TmpString[TmpString.length()-1] == '\n');
+TmpString = TmpString.before((int)(TmpString.length()-1));
+IO->PutS(TmpString+EOL);
+
+IO->PutS("//");
+for(i=IO->GetColNo(); i<IO->GetLineWrap(); i++)
+   { IO->PutC('/');
+    } /* rof */
+IO->PutNL();
+
+if ((SIR_Error = IO->Check()))
+   { return(SIR_Error);
     } /* fi */
 
+
 if (! WriteNotes)
-   { fputs("\n// Note: Notes are suppressed in this file.\n", File);
-     CurrLineNo += 2;
+   { IO->PutNL();
+     IO->PutS("// Note: Notes are suppressed in this file." EOL);
     } /* fi */
 if (! WriteLines)
-   { CurrLineNo += 2;
-     fputs("\n// Note: Line infos are suppressed in this file.\n", File);
+   { IO->PutNL();
+     IO->PutS("// Note: Line infos are suppressed in this file." EOL);
     } /* fi */
 
-fprintf(File, "\n#include \"%s\"\n", GL_GetFileFromPath(Headername));
-CurrLineNo += 2;
+IO->PutNL();
+IO->PrintF("#include \"%s\""EOL, GL_GetFileFromPath(Headername));
 
-SIR_LineInfo::InitWriteSC(WriteLines,	/* init for new file */
-			Filename, CurrLineNo);
+SIR_LineInfo::InitWriteSC(WriteLines, Filename);
 
 if (Symbols)			/* translate all global definitions */
-   { if ((SIR_Error = Symbols->WriteCC(File, WriteNotes)))
+   { if ((SIR_Error = Symbols->WriteCC(IO, WriteNotes)))
 	{ return(SIR_Error);
 	 } /* fi */
     } /* fi */
 
 if (  (NeedForRangeChecker(this))
     &&(!(Symbols->FindLocally(SIR_CXX_RANGE_NAME))))
-   { if ((SIR_Error = DefineRangeChecker(File)))
+   { if ((SIR_Error = DefineRangeChecker(IO)))
 	{ return(SIR_Error);
 	 } /* fi */
     } /* esle */
 
-fprintf(File,
-	"\n\n"
-	"///////////////////////////////////////////////////////////"
-								"///////////\n"
-	"// End of file %s\n"
-	"///////////////////////////////////////////////////////////"
-								"///////////\n",
-	GL_GetFileFromPath(Filename));
+if (IO->GetColNo() > 0)
+   { IO->PutNL();
+    } /* fi */
+IO->PutNL();
+IO->PutS("//");
+for(i=IO->GetColNo(); i<IO->GetLineWrap(); i++)
+   { IO->PutC('/');
+    } /* rof */
+IO->PutNL();
+IO->PutS("// End of file ");
+IO->PutS(GL_GetFileFromPath(Filename));
+IO->PutNL();
+IO->PutS("//");
+for(i=IO->GetColNo(); i<IO->GetLineWrap(); i++)
+   { IO->PutC('/');
+    } /* rof */
+IO->PutNL();
 
-if (ferror(File))
-   { SIR_ErrMsg.form("Writing to file failed%s",
-			GL_SystemErrorMessage());
-     return(SIR_ERROR_WRITING_SPECC_FILE);
+if ((SIR_Error = IO->Check()))
+   { return(SIR_Error);
     } /* fi */
 
 return(SIR_ERROR_NO_ERROR);	/* successful */
@@ -816,12 +890,10 @@ return(SIR_ERROR_NO_ERROR);	/* successful */
 
 
 	/* none */
-
-
 /*** internal functions *************************************************/
 
 
-BOOL NeedForRangeChecker(	/* checks whether range checker is needed */
+static bool NeedForRangeChecker( /* checks whether range checker is needed */
 	sir_design	*Design)
 {
 
@@ -832,24 +904,24 @@ if (SIR_ERROR_FLAG_YES == Design->DFS_ForAllStatements(
 					&SIR_Statement::NeedsRangeChecker,
 					(sir_stmnt_marg) NULL))
    { SIR_Error = SIR_ERROR_NO_ERROR;	/* clear the fake error */
-     return(TRUE);
+     return(true);
     } /* fi */
 
 assert(SIR_Error == SIR_ERROR_FLAG_NO);
 assert(SIR_Error == SIR_ERROR_NO_ERROR);
 
-return(FALSE);	/* don't need it */
+return(false);	/* don't need it */
 
 } /* end of NeedForRangeChecker */
 
 
-ERROR DeclareRangeChecker(	/* generate declaration of range checker */
-	FILE		*File)
+static ERROR DeclareRangeChecker( /* generate declaration of range checker */
+	gl_io	*IO)
 {
 
-SIR_LineInfo::WriteNewSC(File, true);
+SIR_LineInfo::WriteNewSC(IO, true);
 
-fputs("void " SIR_CXX_RANGE_NAME "("
+IO->PutS("void " SIR_CXX_RANGE_NAME "("
 	SIR_CXX_TIME_TYPE ", "
 	SIR_CXX_TIME_TYPE ", "
 	SIR_CXX_TIME_TYPE ", "
@@ -859,8 +931,7 @@ fputs("void " SIR_CXX_RANGE_NAME "("
 	"const char *, "
 	"const char *, "
 	"const char *, "
-	"unsigned int);",
-	File);
+	"unsigned int);");
 
 return(SIR_ERROR_NO_ERROR);
 
@@ -906,82 +977,84 @@ return(SIR_ERROR_NO_ERROR);
 #endif /* LONGLONG_USE_SPECC_OUTPUT_ROUTINES */
 
 
-ERROR DefineRangeChecker(	/* generate definition of range checker */
-	FILE		*File)
+static ERROR DefineRangeChecker( /* generate definition of range checker */
+	gl_io	*IO)
 {
 
-SIR_LineInfo::WriteNewSC(File, true);
+SIR_LineInfo::WriteVSPACE(IO);
+SIR_LineInfo::WriteNewSC(IO, true);
 
-fputs(	"void " SIR_CXX_RANGE_NAME "(", File);
-SIR_LineInfo::WriteNL(File);
-fputs(	"\t\t" SIR_CXX_TIME_TYPE " Time1, ", File);
-SIR_LineInfo::WriteNL(File);
-fputs(	"\t\t" SIR_CXX_TIME_TYPE " Time2, ", File);
-SIR_LineInfo::WriteNL(File);
-fputs(	"\t\t" SIR_CXX_TIME_TYPE " Min, ", File);
-SIR_LineInfo::WriteNL(File);
-fputs(	"\t\tbool CheckMin, ", File);
-SIR_LineInfo::WriteNL(File);
-fputs(	"\t\t" SIR_CXX_TIME_TYPE " Max, ", File);
-SIR_LineInfo::WriteNL(File);
-fputs(	"\t\tbool CheckMax, ", File);
-SIR_LineInfo::WriteNL(File);
-fputs(	"\t\tconst char *Label1, ", File);
-SIR_LineInfo::WriteNL(File);
-fputs(	"\t\tconst char *Label2, ", File);
-SIR_LineInfo::WriteNL(File);
-fputs(	"\t\tconst char *File, ", File);
-SIR_LineInfo::WriteNL(File);
-fputs(	"\t\tunsigned int Line)", File);
-SIR_LineInfo::Add2Tabulator(1);
-SIR_LineInfo::WriteNL(File);
-fputc(	'{', File);
-SIR_LineInfo::WriteNL(File);
+IO->PutS("void " SIR_CXX_RANGE_NAME "(");
+IO->TabStepUp();
+SIR_LineInfo::WriteNL(IO);
+IO->PutS(SIR_CXX_TIME_TYPE " Time1, ");
+SIR_LineInfo::WriteNL(IO);
+IO->PutS(SIR_CXX_TIME_TYPE " Time2, ");
+SIR_LineInfo::WriteNL(IO);
+IO->PutS(SIR_CXX_TIME_TYPE " Min, ");
+SIR_LineInfo::WriteNL(IO);
+IO->PutS("bool CheckMin, ");
+SIR_LineInfo::WriteNL(IO);
+IO->PutS(SIR_CXX_TIME_TYPE " Max, ");
+SIR_LineInfo::WriteNL(IO);
+IO->PutS("bool CheckMax, ");
+SIR_LineInfo::WriteNL(IO);
+IO->PutS("const char *Label1, ");
+SIR_LineInfo::WriteNL(IO);
+IO->PutS("const char *Label2, ");
+SIR_LineInfo::WriteNL(IO);
+IO->PutS("const char *File, ");
+SIR_LineInfo::WriteNL(IO);
+IO->PutS("unsigned int Line)");
+SIR_LineInfo::WriteNL(IO, true, false); // IO->PutC('{');
+SIR_LineInfo::WriteNL(IO);
 
 #ifdef LONGLONG_USE_SPECC_OUTPUT_ROUTINES
-fputs(	"char BuffMin[40],BuffMax[40],BuffT2[40],BuffT1[40],BuffT[40];", File);
-SIR_LineInfo::WriteNL(File);
+IO->PutS("char BuffMin[40],BuffMax[40],BuffT2[40],BuffT1[40],BuffT[40];");
+SIR_LineInfo::WriteNL(IO);
 #endif /* LONGLONG_USE_SPECC_OUTPUT_ROUTINES */
 
-fputs(	"if (  ((CheckMin) && (Time2 - Time1 < Min))", File);
-SIR_LineInfo::WriteNL(File);
-fputs(	"    ||((CheckMax) && (Time2 - Time1 > Max)))", File);
-SIR_LineInfo::WriteNL(File);
-fputs(	"   { " SIR_SIM_ABORT "(", File);
-SIR_LineInfo::WriteNL(File);
-fputs(	"\t\"ERROR:\\tRange check failed in line %d\\n\"", File);
-SIR_LineInfo::WriteNL(File);
-fputs(	"\t\"\\tin file \\\"%s\\\":\\n\"", File);
-SIR_LineInfo::WriteNL(File);
-fputs(	"\t\"\\trange(%s; %s; " FMT_LLD "; " FMT_LLD ") is "
-		FMT_LLD " - " FMT_LLD " = " FMT_LLD ".\\n\"", File);
-SIR_LineInfo::WriteNL(File);
-fputs(	"\t\"Simulation aborted.\\n\",", File);
-SIR_LineInfo::WriteNL(File);
-fputs(	"\tLine, File,", File);
-SIR_LineInfo::WriteNL(File);
+IO->PutS("if (((CheckMin)&&(Time2-Time1 < Min))"
+	" || ((CheckMax)&&(Time2-Time1 > Max)))");
+IO->TabStepUp();
+SIR_LineInfo::WriteNL(IO, true, false); // IO->PutC('{');
+IO->PutS(SIR_SIM_ABORT "(");
+IO->TabStepUp();
+SIR_LineInfo::WriteNL(IO);
+IO->PutS("\"ERROR:\\tRange check failed in line %d\\n\"");
+SIR_LineInfo::WriteNL(IO);
+IO->PutS("\"\\tin file \\\"%s\\\":\\n\"");
+SIR_LineInfo::WriteNL(IO);
+IO->PutS("\"\\trange(%s; %s; " FMT_LLD "; " FMT_LLD ") is "
+		FMT_LLD " - " FMT_LLD " = " FMT_LLD ".\\n\"");
+SIR_LineInfo::WriteNL(IO);
+IO->PutS("\"Simulation aborted.\\n\",");
+SIR_LineInfo::WriteNL(IO);
+IO->PutS("Line, File,");
+SIR_LineInfo::WriteNL(IO);
 
 #ifdef LONGLONG_USE_SPECC_OUTPUT_ROUTINES
 
 				/* 'time' is 'unsigned long long' */
-fputs(	"\tLabel1, Label2, ull2str(10,&BuffMin[39],Min),"
-			" ull2str(10,&BuffMax[39],Max),", File);
-SIR_LineInfo::WriteNL(File);
-fputs(	"\tull2str(10,&BuffT2[39],Time2), ull2str(10,&BuffT1[39],Time1),"
-			" ull2str(10,&BuffT[39],Time2 - Time1)); }", File);
-SIR_LineInfo::WriteNL(File);
+IO->PutS("Label1, Label2, ull2str(10,&BuffMin[39],Min),"
+			" ull2str(10,&BuffMax[39],Max),");
+SIR_LineInfo::WriteNL(IO);
+IO->PutS("ull2str(10,&BuffT2[39],Time2), ull2str(10,&BuffT1[39],Time1),"
+			" ull2str(10,&BuffT[39],Time2 - Time1));");
 
 #else /* not LONGLONG_USE_SPECC_OUTPUT_ROUTINES */
 
-fputs(	"\tLabel1, Label2, Min, Max,", File);
-SIR_LineInfo::WriteNL(File);
-fputs(	"\tTime2, Time1, Time2 - Time1); }", File);
-SIR_LineInfo::WriteNL(File);
+IO->PutS("Label1, Label2, Min, Max,");
+SIR_LineInfo::WriteNL(IO);
+IO->PutS("Time2, Time1, Time2 - Time1);");
 
 #endif /* LONGLONG_USE_SPECC_OUTPUT_ROUTINES */
 
-fputc(	'}', File);
-SIR_LineInfo::Add2Tabulator(-1);
+IO->TabStepDown();
+SIR_LineInfo::WriteNL(IO, false, true); // IO->PutC('}');
+IO->TabStepDown();
+SIR_LineInfo::WriteNL(IO, false, true); // IO->PutC('}');
+IO->TabStepDown();
 
 return(SIR_ERROR_NO_ERROR);
 
