@@ -4,10 +4,12 @@
 /* Author: Rainer Doemer			first version: 02/10/98 */
 /************************************************************************/
 
-/* last update: 11/21/01 */
+/* last update: 09/26/06 */
 
 /* modifications: (most recent first)
  *
+ * 09/26/06 PC  Adjustments for scrc 2.1
+ * 06/15/04 PC  Adjustments for scrc 2.0
  * 11/21/01 RD	took out default arguments from function definitions
  * 05/31/01 RD	eliminated level 2 of SIR API
  * 05/25/01 RD	eliminated support for binary SIR files (import/export)
@@ -231,7 +233,7 @@ sir_import *SIR_ImportList::Insert(		/* insert a new entry */
 {
 
 if (Find(ImportName))
-   { assert(FALSE);	/* already exists! */
+   { assert(false);	/* already exists! */
     } /* fi */
 
 if (Curr())
