@@ -4,10 +4,14 @@
 /* Author: Rainer Doemer			first version: 09/24/97 */
 /************************************************************************/
 
-/* last update: 06/15/04 */
+/* last update: 09/26/06 */
 
 /* modifications: (most recent first)
  *
+ * 09/26/06 PC  Adjustments for scrc 2.1
+ * 06/06/05 RD	reorganized and renamed global type names
+ * 05/31/05 RD	added support for PE-specific memory layout (SIR_MemPara)
+ * 05/23/05 RD	added a type-cast to avoid warning in MS compiler
  * 06/15/04 PC  Adjustments for scrc 2.0
  * 12/04/03 HY	added support for HAVE_METHOD
  * 10/28/03 RD	adjusted for new member ParentStmnt in SIR_Statement
@@ -709,7 +713,8 @@ sir_constant	*MyConst;
 sir_expression	*MyExpression;
 sir_expressions	*MyArgs;
 sir_label	*MyLabel;
-GL_HANDLE	*IO;
+FILE		*FileHandle;
+gl_io		*IO;
 ERROR		Error;
 
 
@@ -829,7 +834,7 @@ MyType = Design1->Types->FindOrInsert(	/* finally build the method type */
 		Design1->Types->FindOrInsert(	/* return type 'int' */
 			SIR_TYPE_INT),
 		ArgTypes,
-		0, FALSE, FALSE, SIR_PORT_NONE,
+		0, false, false, SIR_PORT_NONE,
 		MySymbol);
 MyScope = new SIR_Symbols(		/* create a scope for parameters */
 		LocalScope,		/* (parent is behavior scope) */
