@@ -4,10 +4,22 @@
 /* Author: Rainer Doemer			first version: 10/08/97 */
 /************************************************************************/
 
-/* last update: 11/21/01 */
+/* last update: 09/26/06 */
 
 /* modifications: (most recent first)
  *
+ * 09/26/06 PC  Adjustments for scrc 2.1
+ * 06/03/05 RD	reorganized and renamed global type names
+ * 04/13/05 RD	bug fix: WriteIntro() argument was pointer instead of bool
+ *		(reported by MS compiler)
+ * 06/15/04 PC  Adjustments for scrc 2.0
+ * 02/13/03 RD	added support for notes at fsmd states
+ * 09/19/02 RD	support for adding source code info during code generation
+ * 06/21/02 RD	bug fix: SIR_Notes::Merge() missed some pointer unaliasing
+ * 05/20/02 RD	removed redefined default arguments from a constructor
+ * 01/16/02 RD	added comment in front of generated global annotations
+ * 01/15/02 RD	made SIR_Note inherit from SIR_Node to store location info
+ * 01/14/02 RD	added support for composite annotations
  * 11/21/01 RD	took out default arguments from function definitions
  * 11/08/01 RD	switched code generation to use GL_IO layer
  * 05/31/01 RD	eliminated level 2 of SIR API
@@ -31,9 +43,19 @@
 
 /*** internal type declarations *****************************************/
 
-
 	/* (none) */
 
+/*** prototypes of internal functions ***********************************/
+
+
+#ifdef SIR_SORT_CODE_BY_LINEINFO
+static ERROR WriteOrderedNoteSC(	/* generate sorted source code */
+	sir_notes	*NotesList,
+	sir_note	*Current,
+	gl_io		*IO,
+	bool		CplusplusMode);
+#endif /* SIR_SORT_CODE_BY_LINEINFO */
+
 
 /*** class implementations **********************************************/
 
@@ -48,40 +70,52 @@
 
 SIR_Note::SIR_Note(			/* constructor #1 */
 	const char	*Name,
-	sir_constant	*Content,
+	sir_initializer	*Contents,
 	sir_symbol	*Symbol /* = NULL */,	/* max. one */
 	sir_usertype	*UserType /* = NULL */,	/* of these */
-	sir_label	*Label /* = NULL */)	/* not NULL */
+	sir_label	*Label /* = NULL */,	/* can be   */
+	sir_fsmdstate	*State /* = NULL */,	/* not NULL */
+	unsigned int	Line /* = 0 */,
+	sir_fileinfo	*FileInfo /* = NULL */) :
+		SIR_Node(Line, FileInfo)
 {
 
 assert(Name != NULL);
 assert(*Name != 0);
-assert(Content != NULL);
-assert((Symbol == NULL) || ((UserType == NULL) && (Label == NULL)));
-assert((UserType == NULL) || ((Symbol == NULL) && (Label == NULL)));
-assert((Label == NULL) || ((Symbol == NULL) && (UserType == NULL)));
+assert(Contents != NULL);
+assert((Symbol == NULL) ||
+	((UserType == NULL) && (Label == NULL) && (State == NULL)));
+assert((UserType == NULL) ||
+	((Symbol == NULL) && (Label == NULL) && (State == NULL)));
+assert((Label == NULL) ||
+	((Symbol == NULL) && (UserType == NULL) && (State == NULL)));
+assert((State == NULL) ||
+	((Symbol == NULL) && (UserType == NULL) && (Label == NULL)));
 
 SIR_Note::Name		= Name;
-SIR_Note::Content	= Content;
+SIR_Note::Contents	= Contents;
 SIR_Note::Symbol	= Symbol;
 SIR_Note::UserType	= UserType;
 SIR_Note::Label		= Label;
+SIR_Note::State		= State;
 SIR_Note::Imported	= NULL;
 
 } /* end of SIR_Note::SIR_Note #1 */
 
 
 SIR_Note::SIR_Note(			/* constructor #3 (duplicator) */
-	sir_note	*Original)
+	sir_note	*Original) :
+		SIR_Node(Original)
 {
 
 SIR_Note::Name		= Original->Name;
-SIR_Note::Content	= Original->Content ?
-				new SIR_Constant(Original->Content) :
+SIR_Note::Contents	= Original->Contents ?
+				new SIR_Initializer(Original->Contents) :
 				NULL;
 SIR_Note::Symbol	= Original->Symbol;
 SIR_Note::UserType	= Original->UserType;
 SIR_Note::Label		= Original->Label;
+SIR_Note::State		= Original->State;
 SIR_Note::Imported	= Original->Imported;
 
 } /* end of SIR_Note::SIR_Note #3 */
@@ -90,7 +124,7 @@ SIR_Note::Imported	= Original->Imported;
 SIR_Note::~SIR_Note(void)		/* destructor */
 {
 
-delete Content;
+delete Contents;
 
 } /* end of SIR_Note::~SIR_Note */
 
@@ -100,10 +134,12 @@ ERROR SIR_Note::DFS_ForAllNodes(	/* iterator for over all nodes */
 	sir_node_marg	MemberFctArg)
 {
 
-/* this is not a node, but there are nodes below */
+if ((SIR_Error = (this->*MemberFct)(MemberFctArg)))	/* process this node */
+   { return(SIR_Error);
+    } /* fi */
 
-if (Content)
-   { if ((SIR_Error = Content->DFS_ForAllNodes(MemberFct, MemberFctArg)))
+if (Contents)
+   { if ((SIR_Error = Contents->DFS_ForAllNodes(MemberFct, MemberFctArg)))
 	{ return(SIR_Error);
 	 } /* fi */
     } /* fi */
@@ -182,10 +218,9 @@ return(SIR_ERROR_NO_ERROR);
 
 } /* end of SIR_Note::RemoveDoubleImport */
 
-
 const char *SIR_Note::Print(		/* print ASCII definition */
-	string		*Buffer,	/* (Buffer must be initialized) */
-	BOOL		CplusplusMode /* = FALSE */)
+	gl_string		*Buffer,	/* (Buffer must be initialized) */
+	bool		CplusplusMode /* = false */)
 {
 
 assert(Buffer != NULL);
@@ -209,10 +244,21 @@ if (Label)
    { *Buffer += Label->LabelName;
      *Buffer += '.';
     } /* fi */
+// note: since labels and fsmd-states are in independent
+//       naming scopes, we attach annotations to labels
+//       by using the label prefix; for states, however,
+//       we use the current scope for attaching annotations;
+//       thus, we don't print a prefix for state annotations
+//       (unless in C++ mode where it's more readable)
+if (  (State)
+    &&(CplusplusMode))
+   { *Buffer += State->StateName;
+     *Buffer += '.';
+    } /* fi */
 
 *Buffer += Name;
 *Buffer += " = ";
-*Buffer += Content->Print();
+*Buffer += Contents->Print();
 *Buffer += ';';
 
 if (CplusplusMode)
@@ -225,19 +271,19 @@ return(Buffer->chars());
 
 
 ERROR SIR_Note::WriteSC(		/* (re-) generates SpecC source code */
-	GL_HANDLE	*IO,
-	BOOL		CplusplusMode /* = FALSE */)
+	gl_io		*IO,
+	bool		CplusplusMode /* = false */)
 {
 
-assert(Content != NULL);
+assert(Contents != NULL);
 
-if (Content->LineInfo)
-   { if ((SIR_Error = Content->LineInfo->WriteSC(IO, true)))
+if (LineInfo)
+   { if ((SIR_Error = LineInfo->WriteSC(IO, true)))
 	{ return(SIR_Error);
 	 } /* fi */
     } /* fi */
 else
-   { SIR_LineInfo::WriteNewSC(IO, true);
+   { SIR_LineInfo::WriteNewSC(IO, true, false, &LineInfo);
     } /* esle */
 
 if (CplusplusMode)
@@ -258,10 +304,23 @@ if (Label)
    { IO->PutS(Label->LabelName);
      IO->PutC('.');
     } /* fi */
+// note: since labels and fsmd-states are in independent
+//       naming scopes, we attach annotations to labels
+//       by using the label prefix; for states, however,
+//       we use the current scope for attaching annotations;
+//       thus, we don't print a prefix for state annotations
+//       (unless in C++ mode where it's more readable)
+if (  (State)
+    &&(CplusplusMode))
+   { IO->PutS(State->StateName);
+     IO->PutC('.');
+    } /* fi */
 
 IO->PutS(Name);
 IO->PutS(" = ");
-IO->PutS(Content->Print());
+if ((SIR_Error = Contents->WriteSC(IO, CplusplusMode)))
+   { return(SIR_Error);
+    } /* fi */
 IO->PutC(';');
 
 if (CplusplusMode)
@@ -277,23 +336,6 @@ return(SIR_ERROR_NO_ERROR);
 } /* end of SIR_Note::WriteSC */
 
 
-void SIR_Note::Touch(		/* touch it (remove source information) */
-	sir_design	*Design)
-{
-
-assert(Design != NULL);
-
-if (Imported)		/* remove import information */
-   { Imported->TakeOverAndRemove(Design);
-    } /* fi */
-
-if (Content)
-   { Content->Strip();	/* remove source code info */
-    } /* fi */
-
-} /* end of SIR_Note::Touch */
-
-
 	/*****************/
 	/*** SIR_Notes ***/
 	/*****************/
@@ -428,9 +470,17 @@ while(Curr())
 				NewNote->Label->LabelName.chars());
 			} /* fi */
 		    else
-		       { SIR_ErrMsg.form(
+		       { if (NewNote->State)
+			    { SIR_ErrMsg.form(
+				"Annotation '%s' at state '%s' specified twice",
+				NewNote->Name.chars(),
+				NewNote->State->StateName.chars());
+			     } /* fi */
+			 else
+			    { SIR_ErrMsg.form(
 				"Annotation '%s' at top level specified twice",
 				NewNote->Name.chars());
+			     } /* esle */
 			} /* esle */
 		   } /* esle */
 	      } /* esle */
@@ -495,6 +545,19 @@ else
    { if (*Notes2)
 	{ *Notes1 = *Notes2;	/* just cut and paste */
 	  *Notes2 = NULL;
+	  /* bug fix: automatic unaliasing was missing (06/21/02, RD) */
+	  Curr = (*Notes1)->First();
+	  while(Curr)
+	     { if (  (Curr->Symbol)
+		   &&(Curr->Symbol->Alias))
+		  { Curr->Symbol = Curr->Symbol->Alias;
+		   } /* fi */
+	       if (  (Curr->UserType)
+		   &&(Curr->UserType->Alias))
+		  { Curr->UserType = Curr->UserType->Alias;
+		   } /* fi */
+	       Curr = Curr->Succ();
+	      } /* elihw */
 	 } /* fi */
      else
 	{ /* nothing to do */
@@ -507,10 +570,10 @@ return(SIR_ERROR_NO_ERROR);
 
 
 const char *SIR_Notes::Print(		/* print SpecC code (#1) */
-	BOOL		DoIt,
-	BOOL		CplusplusMode /* = FALSE */)
+	bool		DoIt,
+	bool		CplusplusMode /* = false */)
 {
-static string	Buffer;
+static gl_string Buffer;
 sir_note	*Note;
 
 if (! DoIt)
@@ -534,17 +597,64 @@ return(Buffer.chars());
 
 
 ERROR SIR_Notes::WriteSC(		/* (re-) generates SpecC source code */
-	GL_HANDLE	*IO,
-	BOOL		DoIt,
-	BOOL		CplusplusMode /* = FALSE */)
+	gl_io		*IO,
+	bool		DoIt,
+	bool		CplusplusMode /* = false */)
 {
 sir_note	*Note;
+bool		AtTopLevel;
 
 if (! DoIt)
    { return(SIR_ERROR_NO_ERROR);
     } /* fi */
 
+/* (0) Global annotations */
+
+if (!(Note = First()))
+   { return(SIR_ERROR_NO_ERROR);
+    } /* fi */
+AtTopLevel = (  (Note->Symbol == NULL)
+	      &&(Note->UserType == NULL)
+	      &&(Note->Label == NULL)
+	      &&(Note->State == NULL));
+if (AtTopLevel)
+   { bool PutIntro = true;
+     bool PutSeparator = false;
+#ifdef SIR_PUT_COMMENTS_IN_SOURCE
+     bool PutComment = true;
+#else /* !SIR_PUT_COMMENTS_IN_SOURCE */
+     bool PutComment = false;
+#endif /* SIR_PUT_COMMENTS_IN_SOURCE */
+
+     SIR_Symbols::WriteIntro(IO, false,
+			&PutIntro, &PutSeparator, &PutComment,
+			"global annotations");
+    } /* fi */
+
+#ifdef SIR_SORT_CODE_BY_LINEINFO	/* output notes in LineInfo order */
+
+#ifndef NDEBUG
+while(Note)
+   { assert(Note->Color == SIR_WHITE);	/* assure clean start */
+     Note = Note->Succ();
+    } /* elihw */
 Note = First();
+#endif /* NDEBUG */
+while(Note)
+   { if ((SIR_Error = WriteOrderedNoteSC(this, Note, IO, CplusplusMode)))
+	{ return(SIR_Error);
+	 } /* fi */
+     Note = Note->Succ();
+    } /* elihw */
+Note = First();
+while(Note)
+   { assert(Note->Color == SIR_RED);	/* assure work is done */
+     Note->Color = SIR_WHITE;
+     Note = Note->Succ();
+    } /* elihw */
+
+#else /* !SIR_SORT_CODE_BY_LINEINFO */	/* output notes in alphabetical order */
+
 while(Note)
    { if ((SIR_Error = Note->WriteSC(IO, CplusplusMode)))
 	{ return(SIR_Error);
@@ -552,9 +662,60 @@ while(Note)
      Note = Note->Succ();
     } /* elihw */
 
+#endif /* SIR_SORT_CODE_BY_LINEINFO */
+
+if (AtTopLevel)
+   { SIR_LineInfo::WriteNL(IO);
+    } /* fi */
+
 return(SIR_ERROR_NO_ERROR);
 
 } /* end of SIR_Notes::WriteSC */
 
 
+/*** internal functions *************************************************/
+
+
+#ifdef SIR_SORT_CODE_BY_LINEINFO
+
+static ERROR WriteOrderedNoteSC(	/* generate sorted source code */
+	sir_notes	*NotesList,	/* (recursively) */
+	sir_note	*Current,
+	gl_io		*IO,
+	bool		CplusplusMode)
+{
+sir_note	*Note;
+
+if (Current->Color == SIR_RED)		/* already written? */
+   { return(SIR_ERROR_NO_ERROR);
+    } /* fi */
+assert(Current->Color != SIR_YELLOW);	/* writing still in progress */
+
+Current->Color = SIR_YELLOW;	/* start working on this one */
+
+Note = NotesList->First();
+while(Note)
+   { if (  (Note->Color != SIR_RED)
+	 &&(0 > SIR_Node::CmpLineInfo(Note, Current)))
+	{ if ((SIR_Error = WriteOrderedNoteSC(NotesList, Note,
+						IO, CplusplusMode)))
+	     { return(SIR_Error);
+	      } /* fi */
+	 } /* fi */
+     Note = Note->Succ();
+    } /* elihw */
+
+if ((SIR_Error = Current->WriteSC(IO, CplusplusMode)))
+   { return(SIR_Error);
+    } /* fi */
+
+Current->Color = SIR_RED;	/* work is done for this one */
+
+return(SIR_ERROR_NO_ERROR);
+
+} /* end of WriteOrderedNoteSC */
+
+#endif /* SIR_SORT_CODE_BY_LINEINFO */
+
+
 /* EOF Note.cc */
