@@ -4,10 +4,12 @@
 /* Author: Rainer Doemer			first version: 06/24/98 */
 /************************************************************************/
 
-/* last update: 06/15/04 */
+/* last update: 09/26/06 */
 
 /* modifications: (most recent first)
  *
+ * 09/26/06 PC  Adjustments for scrc 2.1
+ * 06/03/05 RD	reorganized and renamed global type names
  * 06/15/04 PC  Adjustments for scrc 2.0
  * 03/14/03 RD	added C++ code generation for 'signal' and 'buffered' types
  * 10/18/02 RD	handle basics of port maps with SIR_TYPE_SIGNAL, SIR_TYPE_BUFFER
@@ -152,13 +154,13 @@ if (BitSlices)
 
 
 void SIR_PortMap::WriteCC_Decl(		/* generates C++ source code (part 1) */
-	GL_HANDLE	*IO,
+	gl_io		*IO,
 	sir_type	*PortType,
 	int		*BitBusNum,
 	int		*OpenPortNum,
 	int		*ConstPortNum)
 {
-static string	Buffer;
+static gl_string Buffer;
 sir_bitslice	*BitSlice;
 sir_types	*TypeTable;
 sir_type	*ConstType;
@@ -169,7 +171,7 @@ assert(PortType != NULL);
 if (IsOpenMapping())		/* create dummy for open mappings */
    { SIR_LineInfo::WriteNL(IO);
      Buffer.form(SIR_CXX_OPEN_PORT_NAME, (*OpenPortNum)++);
-     IO->PutS(PortType->PrettyString(&Buffer, FALSE, TRUE, FALSE));
+     IO->PutS(PortType->PrettyString(&Buffer, false, true, false));
      IO->PutC(';');
      return;
     } /* fi */
@@ -193,7 +195,7 @@ while(BitSlice)
 	     { ConstType = TypeTable->FindOrInsert(PortType->Type, ConstType);
 	       assert(ConstType != NULL);
 	      } /* fi */
-	  IO->PutS(ConstType->PrettyString(&Buffer, FALSE, TRUE, FALSE));
+	  IO->PutS(ConstType->PrettyString(&Buffer, false, true, false));
 	  IO->PutC(';');
 	 } /* fi */
      BitSlice = BitSlice->Succ();
@@ -242,8 +244,8 @@ const char *SIR_PortMap::WriteCC_Init(	/* generates C++ source code (part 2) */
 	int		*OpenPortNum,
 	int		*ConstPortNum)
 {
-static string	Buffer;
-string		TmpString,
+static gl_string Buffer;
+gl_string	TmpString,
 		PipedAccess;
 sir_bitslice	*BitSlice;
 sir_type	*MapType;
@@ -288,11 +290,11 @@ while(BitSlice)
 	  if (PortType->Type == SIR_TYPE_BUFFER)
 	     { TmpString.form(SIR_CXX_CONST_PORT_NAME "("
 						SIR_SIM_NO_SIG_NAME ", %s)",
-			(*ConstPortNum)++, BitSlice->Constant->Print(TRUE));
+			(*ConstPortNum)++, BitSlice->Constant->Print(true));
 	      } /* fi */
 	  else
 	     { TmpString.form(SIR_CXX_CONST_PORT_NAME "(%s)",
-			(*ConstPortNum)++, BitSlice->Constant->Print(TRUE));
+			(*ConstPortNum)++, BitSlice->Constant->Print(true));
 	      } /* esle */
 	  Buffer += TmpString;
 	 } /* fi */
@@ -314,7 +316,7 @@ if (! IsDirectlyMappable(PortType))	/* indirect mapping required */
 	     { switch(PortType->Direction)
 		  { case SIR_PORT_NONE:
 		    case SIR_PORT_INOUT:
-		       { assert(FALSE);	/* type checked! */
+		       { assert(false);	/* type checked! */
 			}
 		    case SIR_PORT_IN:
 		       { PipedAccess.form(SIR_SIM_PIPED_IN_ACCESS(
@@ -327,7 +329,7 @@ if (! IsDirectlyMappable(PortType))	/* indirect mapping required */
 			 break;
 			}
 		    default:
-		       { assert(FALSE);	/* bad direction */
+		       { assert(false);	/* bad direction */
 			}
 		   } /* hctiws */
 	      } /* fi */
@@ -396,7 +398,7 @@ else
 
 
 void SIR_PortMap::WriteCC(		/* generates C++ source code (part 3) */
-	GL_HANDLE	*IO,
+	gl_io		*IO,
 	sir_type	*PortType,
 	int		*BitBusNum,
 	int		*OpenPortNum,
@@ -425,7 +427,7 @@ if (IsDirectlyMappable(PortType))	/* determine mapping type */
 	{ switch(PortType->Direction)
 	     { case SIR_PORT_NONE:
 	       case SIR_PORT_INOUT:
-		  { assert(FALSE);	/* type checked! */
+		  { assert(false);	/* type checked! */
 		   }
 	       case SIR_PORT_IN:
 		  { IO->PutS(BitSlice->Symbol->Name);
@@ -440,7 +442,7 @@ if (IsDirectlyMappable(PortType))	/* determine mapping type */
 		    break;
 		   }
 	       default:
-		  { assert(FALSE);	/* bad direction */
+		  { assert(false);	/* bad direction */
 		   }
 	      } /* hctiws */
 	 } /* fi */
@@ -456,7 +458,7 @@ else
 } /* end of SIR_PortMap::WriteCC */
 
 
-BOOL SIR_PortMap::IsOpenMapping(void)	/* determines whether mapping is open */
+bool SIR_PortMap::IsOpenMapping(void)	/* determines whether mapping is open */
 {
 sir_bitslice	*BitSlice;
 
@@ -466,22 +468,22 @@ BitSlice = BitSlices->First();
 assert(BitSlice != NULL);
 if (BitSlice->Symbol)
    { assert(BitSlice->Constant == NULL);
-     return(FALSE);	/* (std.) symbol mapping */
+     return(false);	/* (std.) symbol mapping */
     } /* fi */
 
 if (BitSlice->Constant)
-   { return(FALSE);	/* (at least one) constant mapping */
+   { return(false);	/* (at least one) constant mapping */
     } /* fi */
 
 assert(BitSlice->Succ() == NULL);	/* grammar assures these */
 assert(BitSlice->Pred() == NULL);
 
-return(TRUE);
+return(true);
 
 } /* end of SIR_PortMap::IsOpenMapping */
 
 
-BOOL SIR_PortMap::IsDirectlyMappable(	/* determines port mapping type */
+bool SIR_PortMap::IsDirectlyMappable(	/* determines port mapping type */
 	sir_type	*PortType)
 {
 sir_bitslice	*BitSlice;
@@ -503,7 +505,7 @@ BitSlice = BitSlices->First();
 assert(BitSlice != NULL);
 if (  (! BitSlice->Symbol)
     &&(! BitSlice->Constant))
-   { return(FALSE);	/* open mapping */
+   { return(false);	/* open mapping */
     } /* fi */
 
 if (BitSlice->Symbol)
@@ -524,7 +526,7 @@ else
 if (PortType->TypeClass() != SIR_TYPECLASS_BITVECTOR)
    { assert(BitSlices->NumElements() == 1);
      assert(MapType->TypeClass() != SIR_TYPECLASS_BITVECTOR);
-     return(TRUE);
+     return(true);
     } /* fi */
 
 	/* bitvector mapping */
@@ -533,7 +535,7 @@ assert(MapType->TypeClass() == SIR_TYPECLASS_BITVECTOR);
 assert(PortType->TypeClass() == SIR_TYPECLASS_BITVECTOR);
 
 if (BitSlices->NumElements() != 1)
-   { return(FALSE);	/* concatenated bitslices */
+   { return(false);	/* concatenated bitslices */
     } /* fi */
 
 if (  (MapType->Type == SIR_TYPE_SIGNAL)
@@ -547,7 +549,7 @@ if (  (PortType->Type == SIR_TYPE_SIGNAL)
 
 if (  (MapLeft != MapType->LeftBound)
     ||(MapRight != MapType->RightBound))
-   { return(FALSE);	/* sliced and/or reversed mapping */
+   { return(false);	/* sliced and/or reversed mapping */
     } /* fi */
 
 PortLeft = PortType->LeftBound;
@@ -989,7 +991,7 @@ return(SIR_ERROR_NO_ERROR);
 
 
 ERROR SIR_PortMaps::WriteSC(		/* (re-) generates SpecC source code */
-	GL_HANDLE	*IO)
+	gl_io		*IO)
 {
 sir_portmap	*PortMap;
 sir_bitslice	*BitSlice;
