@@ -4,10 +4,14 @@
 /* Author: Rainer Doemer			first version: 10/07/97 */
 /************************************************************************/
 
-/* last update: 06/15/04 */
+/* last update: 09/26/06 */
 
 /* modifications: (most recent first)
  *
+ * 09/26/06 PC  Adjustments for scrc 2.1
+ * 06/03/05 RD	reorganized and renamed global type names
+ * 05/31/05 RD	added support for PE-specific memory layout (SIR_MemPara)
+ * 10/21/04 RD	_SIZEOFBITCLASS() macro padded for alignment
  * 06/15/04 PC  Adjustments for scrc 2.0
  * 05/14/04 PC  signal class was included into '_specc' scope
  * 12/23/03 RD	bug fix for HAVE_METHOD support: don't update type names
@@ -179,7 +183,7 @@ switch(Type)		/* type-specific checks */
      case SIR_TYPE_ARRAY:
      case SIR_TYPE_METHOD:
      case SIR_TYPE_FUNCTION:
-	{ assert(FALSE);	/* not supported by this constructor */
+	{ assert(false);	/* not supported by this constructor */
 	 }
      case SIR_TYPE_ANY_TYPE:
 	{ assert((LeftBound == 0) && (RightBound == 0));
@@ -192,10 +196,10 @@ switch(Type)		/* type-specific checks */
      case SIR_TYPE_INTERFACE:
      case SIR_TYPE_SIGNAL:
      case SIR_TYPE_BUFFER:
-	{ assert(FALSE);	/* not supported by this constructor */
+	{ assert(false);	/* not supported by this constructor */
 	 }
      default:
-	{ assert(FALSE);	/* bad Type */
+	{ assert(false);	/* bad Type */
 	 }
     } /* hctiws */
 
@@ -251,7 +255,7 @@ switch(Type)		/* type-specific checks */
      case SIR_TYPE_UBIT:
      case SIR_TYPE_VOID:
      case SIR_TYPE_EVENT:
-	{ assert(FALSE);	/* wrong constructor */
+	{ assert(false);	/* wrong constructor */
 	 }
      case SIR_TYPE_POINTER:
 	{ assert(Size == 0);
@@ -260,7 +264,7 @@ switch(Type)		/* type-specific checks */
      case SIR_TYPE_STRUCT:
      case SIR_TYPE_UNION:
      case SIR_TYPE_ENUM:
-	{ assert(FALSE);	/* wrong constructor */
+	{ assert(false);	/* wrong constructor */
 	 }
      case SIR_TYPE_ARRAY:
      case SIR_TYPE_METHOD:
@@ -271,14 +275,14 @@ switch(Type)		/* type-specific checks */
      case SIR_TYPE_BEHAVIOR:
      case SIR_TYPE_CHANNEL:
      case SIR_TYPE_INTERFACE:
-	{ assert(FALSE);	/* wrong constructor */
+	{ assert(false);	/* wrong constructor */
 	 }
      case SIR_TYPE_SIGNAL:
      case SIR_TYPE_BUFFER:
 	{ break;
 	 }
      default:
-	{ assert(FALSE);	/* bad Type */
+	{ assert(false);	/* bad Type */
 	 }
     } /* hctiws */
 
@@ -424,7 +428,7 @@ delete Parameters;
 } /* end of SIR_Type::~SIR_Type */
 
 
-BOOL SIR_Type::IsPort(void)	/* determines if this is a port type */
+bool SIR_Type::IsPort(void)	/* determines if this is a port type */
 {
 
 return(  (Type == SIR_TYPE_INTERFACE)
@@ -433,7 +437,7 @@ return(  (Type == SIR_TYPE_INTERFACE)
 } /* end of SIR_Type::IsPort */
 
 
-BOOL SIR_Type::VoidParameters(void)	/* determines if there are no params. */
+bool SIR_Type::VoidParameters(void)	/* determines if there are no params. */
 {
 
 assert(Parameters != NULL);	/* must be a class or function type */
@@ -441,15 +445,15 @@ assert(Parameters != NULL);	/* must be a class or function type */
 if (  (Parameters->NumElements() == 0)
     ||(  (Parameters->NumElements() == 1)
        &&(Parameters->First()->Type->Type == SIR_TYPE_VOID)))
-   { return(TRUE);
+   { return(true);
     } /* fi */
 
-return(FALSE);
+return(false);
 
 } /* end of SIR_Type::VoidParameters */
 
 
-BOOL SIR_Type::IsConvertableTo(	/* determines possibility of conversion */
+bool SIR_Type::IsConvertableTo(	/* determines possibility of conversion */
 	sir_type	*TargetType)	/* (silent standard conversions) */
 {
 
@@ -483,13 +487,13 @@ switch(Type)
 	       case SIR_TYPE_INTERFACE:
 	       case SIR_TYPE_SIGNAL:
 	       case SIR_TYPE_BUFFER:
-		  { return(FALSE);
+		  { return(false);
 		   }
 	       case SIR_TYPE_ENUM:
 		  { return(UserType == TargetType->UserType);
 		   }
 	       default:
-		  { return(TRUE);
+		  { return(true);
 		   }
 	      } /* hctiws */
 	 }
@@ -510,11 +514,11 @@ switch(Type)
 	       case SIR_TYPE_INTERFACE:
 	       case SIR_TYPE_SIGNAL:
 	       case SIR_TYPE_BUFFER:
-		  { return(FALSE);
+		  { return(false);
 		   }
 	       case SIR_TYPE_POINTER:	/* 0 or 0l is same as (void*)0 */
 	       default:
-		  { return(TRUE);
+		  { return(true);
 		   }
 	      } /* hctiws */
 	 }
@@ -537,10 +541,10 @@ switch(Type)
 	       case SIR_TYPE_INTERFACE:
 	       case SIR_TYPE_SIGNAL:
 	       case SIR_TYPE_BUFFER:
-		  { return(FALSE);
+		  { return(false);
 		   }
 	       default:
-		  { return(TRUE);
+		  { return(true);
 		   }
 	      } /* hctiws */
 	 }
@@ -554,21 +558,21 @@ switch(Type)
 	     { case SIR_TYPE_BOOL:
 	       case SIR_TYPE_INT:
 	       case SIR_TYPE_LONG:
-		  { return(TRUE);
+		  { return(true);
 		   }
 	       case SIR_TYPE_POINTER:
 	       case SIR_TYPE_ARRAY:
 		  { if (  (SubType->Type == TargetType->SubType->Type)
 			||(SubType->Type == SIR_TYPE_VOID)
 			||(TargetType->SubType->Type == SIR_TYPE_VOID))
-		       { return(TRUE);
+		       { return(true);
 			} /* fi */
 		    else
-		       { return(FALSE);
+		       { return(false);
 			} /* esle */
 		   }
 	       default:
-		  { return(FALSE);
+		  { return(false);
 		   }
 	      } /* hctiws */
 	 }
@@ -578,36 +582,36 @@ switch(Type)
 	      ||(TargetType->Type == SIR_TYPE_UNION))
 	     { return(UserType == TargetType->UserType);
 	      } /* fi */
-	  return(FALSE);
+	  return(false);
 	 }
      case SIR_TYPE_METHOD:
      case SIR_TYPE_FUNCTION:
 	{ if (TargetType == this)
-	     { return(TRUE);
+	     { return(true);
 	      } /* fi */
 	  if (  (TargetType->Type == SIR_TYPE_POINTER)
 	      &&(TargetType->SubType == this))
-	     { return(TRUE);
+	     { return(true);
 	      } /* fi */
-	  return(FALSE);
+	  return(false);
 	 }
      case SIR_TYPE_ANY_TYPE:
-	{ return(FALSE);
+	{ return(false);
 	 }
      case SIR_TYPE_BEHAVIOR:
      case SIR_TYPE_CHANNEL:
 	{ if (  (TargetType->Type == SIR_TYPE_INTERFACE)
 	      &&(ClassSymbol->Interfaces->Find(TargetType->ClassSymbol)))
-	     { return(TRUE);
+	     { return(true);
 	      } /* fi */
-	  return(FALSE);
+	  return(false);
 	 }
      case SIR_TYPE_INTERFACE:
 	{ return(TargetType == this);
 	 }
      case SIR_TYPE_SIGNAL:
 	{ if (TargetType->Type == SIR_TYPE_EVENT)
-	     { return(TRUE);
+	     { return(true);
 	      } /* fi */
 	  if (  (TargetType->Type == SIR_TYPE_SIGNAL)
 	      ||(TargetType->Type == SIR_TYPE_BUFFER))
@@ -622,16 +626,16 @@ switch(Type)
 	  return(SubType->IsConvertableTo(TargetType));
 	 }
      default:
-	{ assert(FALSE);	/* bad Type */
+	{ assert(false);	/* bad Type */
 	 }
     } /* hctiws */
 
-return(FALSE);	/* to keep the compiler quiet */
+return(false);	/* to keep the compiler quiet */
 
 } /* end of SIR_Type::IsConvertableTo */
 
 
-BOOL SIR_Type::IsAssignableTo(	/* determines possibility of assignment */
+bool SIR_Type::IsAssignableTo(	/* determines possibility of assignment */
 	sir_type	*TargetType)
 {
 
@@ -668,20 +672,20 @@ switch(Type)
 	       case SIR_TYPE_BEHAVIOR:
 	       case SIR_TYPE_CHANNEL:
 	       case SIR_TYPE_INTERFACE:
-		  { return(FALSE);
+		  { return(false);
 		   }
 	       case SIR_TYPE_SIGNAL:
 	       case SIR_TYPE_BUFFER:
 		  { return(IsAssignableTo(TargetType->SubType));
 		   }
 	       default:
-		  { return(TRUE);
+		  { return(true);
 		   }
 	      } /* hctiws */
 	 }
      case SIR_TYPE_VOID:
      case SIR_TYPE_EVENT:
-	{ return(FALSE);
+	{ return(false);
 	 }
      case SIR_TYPE_STRUCT:
      case SIR_TYPE_UNION:
@@ -693,11 +697,11 @@ switch(Type)
 	      ||(TargetType->Type == SIR_TYPE_UNION))
 	     { return(UserType == TargetType->UserType);
 	      } /* fi */
-	  return(FALSE);
+	  return(false);
 	 }
      case SIR_TYPE_ARRAY:
 	{ if (TargetType->Type == SIR_TYPE_POINTER)
-	     { return(TRUE);
+	     { return(true);
 	      } /* fi */
 	  if (  (TargetType->Type == SIR_TYPE_SIGNAL)
 	      ||(TargetType->Type == SIR_TYPE_BUFFER))
@@ -707,38 +711,38 @@ switch(Type)
 	      &&(Size > 0)
 	      &&(Size == TargetType->Size)
 	      &&(SubType->Modified() == TargetType->SubType->Modified()))
-	     { return(TRUE);
+	     { return(true);
 	      } /* fi */
-	  return(FALSE);
+	  return(false);
 	 }		
      case SIR_TYPE_METHOD:
      case SIR_TYPE_FUNCTION:
 	{ if (TargetType->Type == SIR_TYPE_POINTER)
-	     { return(TRUE);	// function is a short-cut to pointer to it
+	     { return(true);	// function is a short-cut to pointer to it
 	      } /* fi */	// (fixed 09/16/02, RD)
-	  return(FALSE);
+	  return(false);
 	 }
      case SIR_TYPE_ANY_TYPE:
      case SIR_TYPE_BEHAVIOR:
      case SIR_TYPE_CHANNEL:
      case SIR_TYPE_INTERFACE:
-	{ return(FALSE);
+	{ return(false);
 	 }
      case SIR_TYPE_SIGNAL:
      case SIR_TYPE_BUFFER:
 	{ return(SubType->IsAssignableTo(TargetType));
 	 }
      default:
-	{ assert(FALSE);	/* bad Type */
+	{ assert(false);	/* bad Type */
 	 }
     } /* hctiws */
 
-return(FALSE);	/* to keep the compiler quiet */
+return(false);	/* to keep the compiler quiet */
 
 } /* end of SIR_Type::IsAssignableTo */
 
 
-BOOL SIR_Type::SubTypeTreeContains(	/* determines special dependency */
+bool SIR_Type::SubTypeTreeContains(	/* determines special dependency */
 	sir_type	*TargetType)	/* (Bug fix 07/01/00, R.D.) */
 {
 sir_type_ptr	*Param;
@@ -746,17 +750,17 @@ sir_type_ptr	*Param;
 assert(TargetType);	/* must be given */
 
 if (SubType == TargetType)
-   { return(TRUE);
+   { return(true);
     } /* fi */
 
 if (Parameters)
    { Param = Parameters->First();
      while(Param)
 	{ if (Param->Type == TargetType)
-	     { return(TRUE);
+	     { return(true);
 	      } /* fi */
 	  if (Param->Type->SubTypeTreeContains(TargetType)) /* recursion */
-	    { return(TRUE);
+	    { return(true);
 	     } /* fi */
 	  Param = Param->Succ();
 	 } /* elihw */
@@ -766,7 +770,7 @@ if (SubType)
    { return(SubType->SubTypeTreeContains(TargetType));	/* recursion */
     } /* fi */
 
-return(FALSE);
+return(false);
 
 } /* end of SIR_Type::SubTypeTreeContains */
 
@@ -796,7 +800,7 @@ switch(TypeType)
 	 }
      case SIR_TYPE_VOID:
      case SIR_TYPE_EVENT:
-	{ assert(FALSE);	/* no equivalent const type */
+	{ assert(false);	/* no equivalent const type */
 	 }
      case SIR_TYPE_POINTER:
 	{ /* assert(SubType->Type == SIR_TYPE_CHAR); */
@@ -804,7 +808,7 @@ switch(TypeType)
 	 }
      case SIR_TYPE_STRUCT:
      case SIR_TYPE_UNION:
-	{ assert(FALSE);	/* no equivalent const type */
+	{ assert(false);	/* no equivalent const type */
 	 }
      case SIR_TYPE_ENUM:
 	{ return(SIR_CONST_INT);
@@ -818,10 +822,10 @@ switch(TypeType)
      case SIR_TYPE_INTERFACE:
      case SIR_TYPE_SIGNAL:
      case SIR_TYPE_BUFFER:
-	{ assert(FALSE);	/* no equivalent const type */
+	{ assert(false);	/* no equivalent const type */
 	 }
      default:
-	{ assert(FALSE);	/* bad Type */
+	{ assert(false);	/* bad Type */
 	 }
     } /* hctiws */
 
@@ -851,7 +855,7 @@ ERROR SIR_Type::Check(		/* performs general legal checks on this type */
 {
 sir_member	*Member;
 sir_type_ptr	*Parameter;
-string		TmpErrMsg;
+gl_string	TmpErrMsg;
 ERROR		Error;
 
 if (IsPort)
@@ -1037,7 +1041,7 @@ switch (Type)
 	  break;
 	 }
      default:
-	{ assert(FALSE);	/* bad Type */
+	{ assert(false);	/* bad Type */
 	 }
     } /* hctiws */
 
@@ -1101,7 +1105,7 @@ switch(Type)
 	 }
      case SIR_TYPE_VOID:
      case SIR_TYPE_EVENT:
-	{ assert(FALSE);
+	{ assert(false);
 	 }
      case SIR_TYPE_POINTER:
 	{ assert(! Initializer->InitList);
@@ -1128,7 +1132,7 @@ switch(Type)
 	 }
      case SIR_TYPE_UNION:
      case SIR_TYPE_ENUM:	// .... no initializer allowed???
-	{ assert(FALSE);
+	{ assert(false);
 	 }
      case SIR_TYPE_ARRAY:
 	{ if (  (  (SubType->Type == SIR_TYPE_CHAR) /* special strings init. */
@@ -1152,7 +1156,7 @@ switch(Type)
      case SIR_TYPE_BEHAVIOR:
      case SIR_TYPE_CHANNEL:
      case SIR_TYPE_INTERFACE:
-	{ assert(FALSE);	/* should not be possible */
+	{ assert(false);	/* should not be possible */
 	 }
      case SIR_TYPE_SIGNAL:
      case SIR_TYPE_BUFFER:
@@ -1160,7 +1164,7 @@ switch(Type)
 	  break;
 	 }
      default:
-	{ assert(FALSE);	/* bad Type */
+	{ assert(false);	/* bad Type */
 	 }
     } /* hctiws */
 
@@ -1325,14 +1329,14 @@ switch(Type)
      case SIR_TYPE_BEHAVIOR:
      case SIR_TYPE_CHANNEL:
      case SIR_TYPE_INTERFACE:
-	{ assert(FALSE);	/* should not be possible */
+	{ assert(false);	/* should not be possible */
 	 }
      case SIR_TYPE_SIGNAL:
      case SIR_TYPE_BUFFER:
 	{ return(SubType->CheckInitializer(Initializer));
 	 }
      default:
-	{ assert(FALSE);	/* bad Type */
+	{ assert(false);	/* bad Type */
 	 }
     } /* hctiws */
 
@@ -1363,7 +1367,7 @@ return(GetTable()->FindOrInsertModifiedType(this,
 
 unsigned int SIR_Type::SizeOf(		/* compute the size of this type */
 	unsigned int	*Alignment /* = NULL */) /* obtain alignment (NULL) */
-//	BOOL		SpecCsize /* = TRUE */)	 /* SpecC or C++ size */
+//	bool		SpecCsize /* = true */)	 /* SpecC or C++ size */
 {
 sir_member	*Member;
 unsigned int	Size,
@@ -1443,7 +1447,9 @@ switch(Type)
 	 }
      case SIR_TYPE_BIT:
      case SIR_TYPE_UBIT:
-	{ if (Alignment)
+	{ unsigned int	padded_size;
+
+	  if (Alignment)
 	     { *Alignment = SIR_ALIGN_BITVECTOR;
 	      } /* fi */
 //
@@ -1461,14 +1467,20 @@ switch(Type)
 //	     { return(_SIZEOFBITCLASS(_BITLEN(LeftBound, RightBound)));
 //	      } /* esle */
 //
-	  return(_SIZEOFBITCLASS(_BITLEN(LeftBound, RightBound)));
+// RD, 10/21/04: _SIZEOFBITCLASS() macro is not padded for alignment!
+	  padded_size = _SIZEOFBITCLASS(_BITLEN(LeftBound, RightBound));
+	  if (padded_size % SIR_ALIGN_BITVECTOR)	// pad size to alignment!
+	     { padded_size += SIR_ALIGN_BITVECTOR -	// (RD, 10/21/04)
+				(padded_size % SIR_ALIGN_BITVECTOR);
+	      } /* fi */
+	  return(padded_size);
 	 }
      case SIR_TYPE_VOID:
-	{ assert(FALSE);	/* should have been caught by type checking */
+	{ assert(false);	/* should have been caught by type checking */
 	 }
      case SIR_TYPE_EVENT:
 	{ 
-//	  assert(SpecCsize == FALSE);	/* in SpecC, sizeof(event) is illegal */
+//	  assert(SpecCsize == false);	/* in SpecC, sizeof(event) is illegal */
 	  if (Alignment)
 	     { *Alignment = SIR_ALIGN_EVENT;
 	      } /* fi */
@@ -1535,7 +1547,7 @@ switch(Type)
 			 break;
 			}
 		    default:
-		       { assert(FALSE);	/* unsupported alignment */
+		       { assert(false);	/* unsupported alignment */
 			}
 		   } /* hctiws */
 	       Size += MSize;		/* add member's size */
@@ -1587,15 +1599,15 @@ switch(Type)
 	 }
      case SIR_TYPE_METHOD:
      case SIR_TYPE_FUNCTION:
-	{ assert(FALSE);	/* should have been caught by type checking */
+	{ assert(false);	/* should have been caught by type checking */
 	 }
      case SIR_TYPE_ANY_TYPE:
-	{ assert(FALSE);	/* should not be possible */
+	{ assert(false);	/* should not be possible */
 	 }
      case SIR_TYPE_BEHAVIOR:
      case SIR_TYPE_CHANNEL:
      case SIR_TYPE_INTERFACE:
-	{ assert(FALSE);	/* should have been caught by type checking */
+	{ assert(false);	/* should have been caught by type checking */
 	 }
      case SIR_TYPE_SIGNAL:
 	{ /* note: this case is used for IPs only! */
@@ -1623,7 +1635,7 @@ switch(Type)
 			 break;
 			}
 		    default:
-		       { assert(FALSE);	/* unsupported alignment */
+		       { assert(false);	/* unsupported alignment */
 			}
 		   } /* hctiws */
 	       Size += MSize;		/* add member's size */
@@ -1649,7 +1661,7 @@ switch(Type)
 		    break;
 		   }
 	       default:
-		  { assert(FALSE);	/* unsupported alignment */
+		  { assert(false);	/* unsupported alignment */
 		   }
 	      } /* hctiws */
 	  Size += MSize;		/* add member's size */
@@ -1689,7 +1701,7 @@ switch(Type)
 			 break;
 			}
 		    default:
-		       { assert(FALSE);	/* unsupported alignment */
+		       { assert(false);	/* unsupported alignment */
 			}
 		   } /* hctiws */
 	       Size += MSize;		/* add member's size */
@@ -1705,7 +1717,7 @@ switch(Type)
 	  return(Size);
 	 }
      default:
-	{ assert(FALSE);	/* bad Type */
+	{ assert(false);	/* bad Type */
 	 }
     } /* hctiws */
 
@@ -1785,7 +1797,7 @@ switch(Type)
 			 break;
 			}
 		    default:
-		       { assert(FALSE);	/* unsupported alignment */
+		       { assert(false);	/* unsupported alignment */
 			}
 		   } /* hctiws */
 	       if (Member == TheMember)
@@ -1795,13 +1807,13 @@ switch(Type)
 	       Size += MSize;		/* add member's size */
 	       Member = Member->Succ();
 	      } /* elihw */
-	  assert(FALSE);	/* TheMember was not found! */
+	  assert(false);	/* TheMember was not found! */
 	 }
      case SIR_TYPE_UNION:
 	{ return(0);	/* offset is the same for every member */
 	 }
      default:	/* anything but struct/union */
-	{ assert(FALSE);	/* bad Type */
+	{ assert(false);	/* bad Type */
 	 }
     } /* hctiws */
 
@@ -1825,7 +1837,7 @@ return(Offset(UserType->Members->Find(Symbol)));
 
 const char *SIR_Type::PrintDirection(	/* prints a SpecC port direction */
 	SIR_DIRECTION	Direction,	/* (eg. "out" or "out ") */
-	BOOL		AppendSpace /* = false */)
+	bool		AppendSpace /* = false */)
 {
 
 switch(Direction)
@@ -1857,7 +1869,7 @@ switch(Direction)
 	      } /* esle */
 	 }
      default:
-	{ assert(FALSE);	/* bad port direction */
+	{ assert(false);	/* bad port direction */
 	 }
     } /* hctiws */
 
@@ -1867,14 +1879,14 @@ return(NULL);	/* never reached */
 
 
 const char *SIR_Type::PrettyString(	/* prints a SpecC type (#1) */
-	string	*Buffer,		/* (Buffer must be initialized */
-	BOOL	IncludeNotes /* = FALSE */,	/* with SymbolName or "" for */
-	BOOL	CplusplusMode /* = FALSE */,	/* abstract types)           */
-	BOOL	UserTypeDefReq /* = FALSE */,
-	BOOL	UseShadowName /* = FALSE */,
+	gl_string	*Buffer,		/* (Buffer must be initialized */
+	bool	IncludeNotes /* = false */,	/* with SymbolName or "" for */
+	bool	CplusplusMode /* = false */,	/* abstract types)           */
+	bool	UserTypeDefReq /* = false */,
+	bool	UseShadowName /* = false */,
 	int	AutoArraySize /* = SIR_UNKNOWN_ARRAY_SIZE */)
 {
-string		Prefix,
+gl_string	Prefix,
 		Postfix;
 int		LastChar;
 
@@ -1910,16 +1922,16 @@ return(Buffer->chars());
 
 
 void SIR_Type::PrettyString(		/* prints a SpecC type (#2) */
-	string	*PrefixReturn,			/* (both Prefix- and Postfix- */
-	string	*PostfixReturn,			/* Return must be initialized */
-	BOOL	IncludeNotes /* = FALSE */,	/* with an empty string ("")  */
-	BOOL	CplusplusMode /* = FALSE */,
-	BOOL	UserTypeDefReq /* = FALSE */,
-	BOOL	UseShadowName /* = FALSE */,
+	gl_string *PrefixReturn,		/* (both Prefix- and Postfix- */
+	gl_string *PostfixReturn,		/* Return must be initialized */
+	bool	IncludeNotes /* = false */,	/* with an empty string ("")  */
+	bool	CplusplusMode /* = false */,
+	bool	UserTypeDefReq /* = false */,
+	bool	UseShadowName /* = false */,
 	int	AutoArraySize /* = SIR_UNKNOWN_ARRAY_SIZE */)
 {
 sir_type_ptr	*Param;
-string		ParamBuffer,
+gl_string	ParamBuffer,
 		TmpString;
 
 assert(PrefixReturn != NULL);
@@ -2174,7 +2186,7 @@ switch(Type)
 	  break;
 	 }
      default:
-	{ assert(FALSE);	/* bad Type */
+	{ assert(false);	/* bad Type */
 	 }
     } /* hctiws */
 
@@ -2266,7 +2278,7 @@ switch(Type)
         { return(SubType->TypeClass());
          }
      default:
-	{ assert(FALSE);	/* bad Type */
+	{ assert(false);	/* bad Type */
 	 }
     } /* hctiws */
 
@@ -2396,7 +2408,7 @@ void SIR_Types::GarbageCollection(void)	/* garbage collector */
 sir_type	*Curr,
 		*Succ;
 #ifdef SIR_DEBUG_TYPE_GARBAGE_COLLECTOR
-string		Buffer;
+gl_string	Buffer;
 #endif /* SIR_DEBUG_TYPE_GARBAGE_COLLECTOR */
 
 #ifdef SIR_DEBUG_TYPE_GARBAGE_COLLECTOR
@@ -2494,7 +2506,7 @@ switch(ConstType)
 				 	NULL, 0, true, false)));
 	 }
      default:
-	{ assert(FALSE);	/* bad ConstType */
+	{ assert(false);	/* bad ConstType */
 	 }
     } /* hctiws */
 
@@ -2745,7 +2757,7 @@ sir_type *SIR_Types::Insert(		/* insert a new entry */
 sir_type	*TypeEntry;
 
 if ((TypeEntry = Find(NewType)))
-   { assert(FALSE);	/* already exists! */
+   { assert(false);	/* already exists! */
     } /* fi */
 
 if (Curr())
@@ -2921,7 +2933,7 @@ switch(Type1->Type)	/* type-specific comparison */
 				IgnoreDirection, XCmpUTypes));	/* recursion */
 	 }
      default:
-	{ assert(FALSE);	/* bad Type */
+	{ assert(false);	/* bad Type */
 	 }
     } /* hctiws */
 
