@@ -4,10 +4,14 @@
 /* Author: Rainer Doemer			first version: 07/02/97 */
 /************************************************************************/
 
-/* last update: 11/15/01 */
+/* last update: 09/26/06 */
 
 /* modifications: (most recent first)
  *
+ * 09/26/06 PC  Adjustments for scrc 2.1
+ * 06/06/05 RD	reorganized and renamed global type names
+ * 06/15/04 PC  Adjustments for scrc 2.0
+ * 01/16/02 RD	added support for parsing (composite) annotations
  * 11/15/01 RD	made grammar selector visible to parser
  * 05/30/01 RD	reduced contents to reference compiler needs
  * 05/04/01 RD	added support for import of SpecC source (.sc) files
@@ -36,7 +40,7 @@
 
 enum PRS_GrammarType		/* supported (sub-) grammars */
 {
-PRS_PARSE_LANGUAGE
+PRS_PARSE_LANGUAGE,
 };
 
 
@@ -69,14 +73,14 @@ PRS_ScannerToken(void);
 class PRS_StringToken	: public PRS_ScannerToken
 {
 public:
-string		CharString;
+gl_string	CharString;
 };
 
 
 class PRS_NameToken	: public PRS_ScannerToken
 {
 public:
-string		Name;
+gl_string	Name;
 sir_symbol	*Symbol;
 };
 
@@ -92,11 +96,11 @@ class PRS_StackItem
 {
 public:
 prs_stackitem	*Next;
-string		NestedFileName;
-string		PreprocName;
+gl_string	NestedFileName;
+gl_string	PreprocName;
 sir_design	*Design;
 unsigned int	LineNo;
-string		FileName;
+gl_string	FileName;
 sir_fileinfo	*FileInfo;
 FILE		*File;
 void		*Buffer;	// (YY_BUFFER_STATE is not available here)
@@ -107,7 +111,7 @@ PRS_StackItem(
 	const char	*PreprocName,
 	sir_design	*Design,
 	unsigned int	LineNo,
-	string		FileName,
+	gl_string	FileName,
 	sir_fileinfo	*FileInfo,
 	FILE		*File,
 	void		*Buffer);
@@ -122,9 +126,15 @@ PRS_StackItem(
 extern sir_design	*PRS_TheDesign;		/* the current design */
 extern sir_symbols	*PRS_CurrScope;		/* the current scope */
 extern unsigned int	PRS_lineno;		/* current line number */
-extern string		PRS_CurrentFile;	/* current filename */
+extern gl_string	PRS_CurrentFile;	/* current filename */
 extern sir_fileinfo	*PRS_CurrFileInfo;	/* current fileinfo */
-extern GL_STRINGLIST	*PRS_ImportDirList;	/* paths to search for import */
+extern gl_stringlist	*PRS_ImportDirList;	/* paths to search for import */
+extern sir_statement	*PRS_FunctionBody;	/* function-body mode */
+extern bool		PRS_StoreFunctionBody;	/* function-body mode */
+extern sir_constant	*PRS_Constant;		/* constant parsing mode */
+extern sir_initializer	*PRS_Annotation;	/* annotation mode */
+extern sir_type		*PRS_Type;		/* type parsing mode */
+extern bool		PRS_ParsingClassBody;	/* class-body mode */
 extern char		*PRS_text;		/* current text */
 extern FILE		*PRS_in;		/* handle for input stream */
 
@@ -151,7 +161,7 @@ extern ERROR PRS_Push(			/* start parsing a nested file */
 	const char	*PreprocName);
 
 extern sir_design *PRS_Pop(	/* resume parsing after importing a file */
-	string		*ImportFileName = NULL);
+	gl_string	*ImportFileName = NULL);
 
 extern void PRS_ClearStack(void);	/* clean up the stack (if any) */
 
