@@ -4,10 +4,26 @@
 /* Author: Rainer Doemer			first version: 03/22/01 */
 /************************************************************************/
 
-/* last update: 06/01/01 */
+/* last update: 06/15/04 */
 
 /* modifications: (most recent first)
  *
+ * 06/15/04 PC  Adjustments for scrc 2.0
+ * 05/15/03 RD	arranged layout of _specc::event to satisfy sizeof() limitation
+ * 03/20/03 RD	added support for signal bitvector ports which are mapped
+ *		onto concatenated slices of bitvector signals
+ *		(i.e. _specc::event::IsNotified(), MappedEvents())
+ * 01/08/03 RD	added support for self-triggering events (_specc::auto_event)
+ * 12/13/02 RD	added support for asynchronous reset of 'buffered' variables
+ * 12/06/02 RD	distinguish 'signal'/'buffered' variables in separate lists
+ * 11/27/02 RD	class redesign: signal _is_ an event (not, _has_ an event);
+ *		also, added support for signal edges in event lists
+ * 06/27/02 RD	refined _specc::event_list::Triggers() to return the match
+ * 06/26/02 RD	added 'wait' statement with AND semantics (specc::wait_and())
+ * 03/28/02 RD	bug fix: removed bad left-over code in _specc::pipe #2
+ * 03/07/02 RD	extended the constructor of 'event_ptr'
+ * 02/27/02 RD	added support for 'signal' and 'buffered' variables
+ * 02/27/02 RD	bug fix: update event lists in event destructor
  * 06/01/01 RD	two bug fixes for bugs showing up on linux
  * 05/25/01 RD	removed code not needed for the SCRC
  * 05/16/01 RD	resolved name space competition with user by moving more
@@ -101,8 +117,8 @@ _specc::queue	_specc::WaitforQueue;	/* the event queue for 'waitfor' */
 					/* ordered by increasing time stamps) */
 
 _specc::queue	_specc::WaitQueue;	/* the queue for 'wait'ing threads */
-					/* (threads in SIM_STATE_WAIT, */
-					/* in no particular order)     */
+					/* (threads in SIM_STATE_WAIT[AND], */
+					/* in no particular order)          */
 
 _specc::queue	_specc::NotifiedQueue;	/* the queue for 'notify'ed threads */
 					/* (threads in SIM_STATE_NOTIFIED, */
@@ -123,6 +139,28 @@ _specc::event_list *_specc::Notify1List	/* list of 'notifyone'ed events */
 			= NULL;
 
 
+// note that there are two lists in order to distinguish 'signal'
+// and 'buffered' variables; 'buffered' variables are listed
+// in BufferedVars, whereas 'signal' variables are listed in SignalVars;
+// this is important for the update-step in the scheduler,
+// where 'signal' variables must be updated only after the
+// general 'buffered' variables, because 'buffered' variables
+// can depend on 'rising' or 'falling' 'signal' variables
+// (but not vice-versa!)
+
+_specc::buffered_base *_specc::BufferedVars /* list of 'buffered' variables */
+			= NULL;
+
+_specc::buffered_base *_specc::SignalVars   /* list of 'signal' variables */
+			= NULL;
+
+
+_specc::auto_event *_specc::FirstAutoEvent	/* list of self-triggering */
+			= NULL;
+_specc::auto_event *_specc::LastAutoEvent	/* events                  */
+			= NULL;
+
+
 	/*** wrapped functions ***/
 
 
@@ -322,14 +360,6 @@ unsigned int	i;
 ThisThread = CurrentThread;
 assert(ThisThread->Status == SIM_STATE_RUNNING);
 
-if (NumStages == 0)
-   { ThisThread->Schedule();	/* empty 'pipe(...)' construct is */
-     return;			/* just a possible task switch    */
-    } /* fi */
-
-assert(FirstStage <= LastStage);	/* don't allow stupid values */
-assert(LastStage <= NumStages);
-
 if (NumStages == 0)		/* empty 'pipe' construct is    */
    { ThisThread->Schedule();	/* just a possible task switch  */
      va_start(Args, LastStage);	/* plus one shift of piped data */
@@ -342,6 +372,9 @@ if (NumStages == 0)		/* empty 'pipe' construct is    */
      return;
     } /* fi */
 
+assert(FirstStage <= LastStage);	/* don't allow stupid values */
+assert(LastStage <= NumStages);
+
 va_start(Args, LastStage);
 for(i=1; i<=NumStages; i++)
    { Fork = va_arg(Args, fork*);
@@ -463,13 +496,13 @@ assert(ThisThread->Status == SIM_STATE_RUNNING);
 } /* end of _specc::waitfor */
 
 
-void _specc::wait(		/* SpecC 'wait' replacement */
-	event		*First,		/* (NULL-terminated list of events) */
+void _specc::wait(		/* SpecC 'wait' replacement (OR semantics) */
+	event_ptr	*First,		/* (NULL-terminated list of events) */
 	...		)
 {
 va_list		Args;
 thread		*ThisThread;
-event		*Event;
+event_ptr	*EventPtr;
 event_ptr	**EventPtrPtr;
 
 ThisThread = CurrentThread;
@@ -478,21 +511,18 @@ assert(ThisThread->Status == SIM_STATE_RUNNING);
 assert(ThisThread->EventList == NULL);	/* event list not in use */
 
 // note:
-//	here, it is tempting to use the variable argument list directly
-//	instead of copying it, but, on the other hand, it is really dirty
-//	if we access data on this thread's stack space with va_arg() from
-//	another thread (is that really portable?); so, to be on the safe
-//	and clean side, the argument list is copied onto the heap;
+//	the caller is assumed to have properly allocated new event_ptr
+//	objects for the argument list; therefore, we just have to
+//	concatenate the arguments and store the list for the time it is used;
+//	after its use, we will delete the list
 
-va_start(Args, First);		/* copy argument list */
-Event = First;
+va_start(Args, First);		/* traverse the argument list */
+EventPtr = First;
 EventPtrPtr = &ThisThread->EventList;
-while(Event)
-   { if (!(*EventPtrPtr = new _specc::event_ptr(Event)))
-	{ fatalError("Cannot build event list", ENOMEM);
-	 } /* fi */
-     EventPtrPtr = &(*EventPtrPtr)->Next;
-     Event = va_arg(Args, event*);
+while(EventPtr)
+   { *EventPtrPtr = EventPtr;
+     EventPtrPtr = &EventPtr->Next;
+     EventPtr = va_arg(Args, event_ptr*);
     } /* elihw */
 va_end(Args);
 
@@ -501,7 +531,7 @@ WaitQueue.Append(ThisThread);	/* create entry in the wait queue */
 ThisThread->Status = SIM_STATE_WAIT;	/* change state to WAIT */
 ThisThread->Schedule();			/* call the scheduler */
 
-delete ThisThread->EventList;	/* delete the copied list again */
+delete ThisThread->EventList;	/* delete the supplied list */
 ThisThread->EventList = NULL;
 
 assert(ThisThread->Status == SIM_STATE_RUNNING);
@@ -509,26 +539,72 @@ assert(ThisThread->Status == SIM_STATE_RUNNING);
 } /* end of _specc::wait */
 
 
-void _specc::notify(		/* SpecC 'notify' replacement */
-	event		*First,		/* (NULL-terminated list of events) */
+void _specc::wait_and(		/* SpecC 'wait' replacement (AND semantics) */
+	event_ptr	*First,		/* (NULL-terminated list of events) */
 	...		)
 {
 va_list		Args;
 thread		*ThisThread;
-event		*Event;
+event_ptr	*EventPtr;
+event_ptr	**EventPtrPtr;
 
 ThisThread = CurrentThread;
 assert(ThisThread->Status == SIM_STATE_RUNNING);
 
+assert(ThisThread->EventList == NULL);	/* event list not in use */
+
+// note:
+//	the caller is assumed to have properly allocated new event_ptr
+//	objects for the argument list; therefore, we just have to
+//	concatenate the arguments and store the list for the time it is used;
+//	the list will be deleted element by element whenever
+//	a listed event is notified (until the list is empty)
+
+va_start(Args, First);		/* copy argument list */
+EventPtr = First;
+EventPtrPtr = &ThisThread->EventList;
+while(EventPtr)
+   { *EventPtrPtr = EventPtr;
+     EventPtrPtr = &EventPtr->Next;
+     EventPtr = va_arg(Args, event_ptr*);
+    } /* elihw */
+va_end(Args);
+
+WaitQueue.Append(ThisThread);	/* create entry in the wait queue */
+
+ThisThread->Status = SIM_STATE_WAITAND;	/* change state to WAITAND */
+ThisThread->Schedule();			/* call the scheduler */
+
+assert(ThisThread->EventList == NULL);	/* must be empty now */
+
+assert(ThisThread->Status == SIM_STATE_RUNNING);
+
+} /* end of _specc::wait_and */
+
+
+void _specc::notify(		/* SpecC 'notify' replacement */
+	event_ptr	*First,		/* (NULL-terminated list of events) */
+	...		)
+{
+va_list		Args;
+thread		*ThisThread;
+event_ptr	*EventPtr;
+
+ThisThread = CurrentThread;
+assert(ThisThread->Status == SIM_STATE_RUNNING);
+
+// note:
+//	the caller is assumed to have properly allocated new event_ptr
+//	objects for the argument list; because we only need the events
+//	inside this function, we will delete the arguments immediately
+//	after their use
+
 va_start(Args, First);		/* include all arguments into the */
-Event = First;			/* list of notified events	  */
-while(Event)
-   { if (! Event->Notified)	/* not notified yet? */
-	{ Event->Notified = true;
-	  Event->Next = NotifiedEvents;	/* insert at the beginning */
-	  NotifiedEvents = Event;
-	 } /* fi */
-     Event = va_arg(Args, event*);
+EventPtr = First;		/* list of notified events	  */
+while(EventPtr)
+   { EventPtr->Event->Notify();
+     delete EventPtr;
+     EventPtr = va_arg(Args, event_ptr*);
     } /* elihw */
 va_end(Args);
 
@@ -549,29 +625,47 @@ va_end(Args);
 
 
 void _specc::notifyone(		/* SpecC 'notifyone' replacement */
-	event		*First,		/* (NULL-terminated list of events) */
+	event_ptr	*First,		/* (NULL-terminated list of events) */
 	...		)
 {
 va_list		Args;
 thread		*ThisThread;
-event		*Event;
+event_ptr	*EventPtr;
 event_ptr	*EventList,
-		**EventPtrPtr;
+		**EventPtrPtr,
+		*MappedEvents;
 event_list	*NewEventList;
 
 ThisThread = CurrentThread;
 assert(ThisThread->Status == SIM_STATE_RUNNING);
 
+// note:
+//	the caller is assumed to have properly allocated new event_ptr
+//	objects for the argument list; therefore, we just have to
+//	concatenate the arguments and store the list for the time it is used;
+//	after its use, we will delete the list
+
 va_start(Args, First);		/* store all arguments into the */
-Event = First;			/* list of 'notifyone'ed events */
+EventPtr = First;		/* list of 'notifyone'ed events */
 EventList = NULL;
 EventPtrPtr = &EventList;
-while(Event)
-   { if (!(*EventPtrPtr = new _specc::event_ptr(Event)))
-	{ fatalError("Cannot build event list", ENOMEM);
+while(EventPtr)
+   { assert(EventPtr->Event != NULL);	// must be valid event
+     assert(EventPtr->Next == NULL);	// must be single element
+     if ((MappedEvents = EventPtr->Event->MappedEvents()))
+	{ /* event is actually a set of mapped bitvector signals */
+	  *EventPtrPtr = MappedEvents;
+	  while(*EventPtrPtr)
+	     { EventPtrPtr = &(*EventPtrPtr)->Next;
+	      } /* elihw */
+	  delete EventPtr;
 	 } /* fi */
-     EventPtrPtr = &(*EventPtrPtr)->Next;
-     Event = va_arg(Args, event*);
+     else
+	{ /* event is an ordinary event or signal */
+	  *EventPtrPtr = EventPtr;
+	  EventPtrPtr = &EventPtr->Next;
+	 } /* esle */
+     EventPtr = va_arg(Args, event_ptr*);
     } /* elihw */
 va_end(Args);
 if (!(NewEventList = new _specc::event_list(EventList)))
@@ -619,33 +713,276 @@ exit(RESULT_FATAL);
 _specc::event::event(void)	/* constructor #1 */
 {
 
+_specc::event::Next		= NULL;
 _specc::event::Notified		= false;
 _specc::event::Triggered	= false;
-_specc::event::Next		= NULL;
 
 } /* end of _specc::event::event */
 
 
 _specc::event::~event(void)	/* destructor */
 {
+event		*Pred;
+event_list	*PrevList,
+		*List;
+event_ptr	*PrevEvent,
+		*Event;
 
-/* nothing to do */
+/* bug fix 02/27/02, RD: remove yourself from the global event lists */
+if (Notified)
+   { if (NotifiedEvents == this)
+	{ NotifiedEvents = Next;
+	 } /* fi */
+     else
+	{ Pred = NotifiedEvents;
+	  while(Pred)
+	     { if (Pred->Next == this)
+		  { Pred->Next = Next;
+		    break;
+		   } /* fi */
+	       Pred = Pred->Next;
+	      } /* elihw */
+	 } /* esle */
+    } /* fi */
+if (Notify1List)
+   { List = Notify1List;
+     PrevList = NULL;
+     while(List)
+	{ Event = List->EventList;
+	  PrevEvent = NULL;
+	  while(Event)
+	     { if (Event->Event == this)
+		  { if (PrevEvent)
+		       { PrevEvent->Next = Event->Next;
+			 Event->Next = NULL;
+			 delete Event;
+			 Event = PrevEvent->Next;
+			} /* fi */
+		    else
+		       { List->EventList = Event->Next;
+			 Event->Next = NULL;
+			 delete Event;
+			 Event = List->EventList;
+			} /* esle */
+		   } /* fi */
+	       else
+		  { PrevEvent = Event;
+		    Event = Event->Next;
+		   } /* esle */
+	      } /* elihw */
+	  if (! List->EventList)
+	     { if (PrevList)
+		  { PrevList->Next = List->Next;
+		    List->Next = NULL;
+		    delete List;
+		    List = PrevList->Next;
+		   } /* fi */
+	       else
+		  { Notify1List = List->Next;
+		    List->Next = NULL;
+		    delete List;
+		    List = Notify1List;
+		   } /* esle */
+	      } /* fi */
+	  else
+	     { PrevList = List;
+	       List = List->Next;
+	      } /* esle */
+	 } /* elihw */
+    } /* fi */
 
 } /* end of _specc::event::~event */
 
 
-void _specc::event::Reset(	/* reset the list of events */
+bool _specc::event::IsNotified(void)	/* check if this has been notified */
+{					/* (to override by signal busses) */
+
+return(Notified);
+
+} /* end of _specc::event::IsNotified */
+
+
+void _specc::event::Notify(		/* notify this event */
 	void)
 {
 
-if (Next)
-   { Next->Reset();	/* use recursion */
+if (! Notified)		/* not notified yet? */
+   { Notified = true;
+     assert(Next == NULL);
+     Next = NotifiedEvents;	/* insert in list (at the beginning) */
+     NotifiedEvents = this;
     } /* fi */
 
-Notified = false;
-Next = NULL;
+} /* end of _specc::event::Notify */
 
-} /* end of _specc::event::Reset */
+
+void _specc::event::UnNotifyAll(	/* un-notify all notified events */
+	void)
+{
+event		*Event,
+		*Succ;
+
+Event = NotifiedEvents;
+while(Event)
+   { Succ = Event->Next;
+     Event->Notified = false;
+     Event->Next = NULL;
+     Event = Succ;
+    } /* elihw */
+
+NotifiedEvents = NULL;
+
+} /* end of _specc::event::UnNotifyAll */
+
+
+bool _specc::event::IsValidEdge(	/* signal edge event filter */
+	EDGE		Edge)		/* (dummy to be overloaded) */
+{
+
+assert(Edge == EDGE_ANY);	// cannot apply 'rising' or 'falling'
+				// to a pure event (type checked)!
+
+return(true);	/* pure event doesn't have edges, every event is valid */
+
+} /* end of _specc::event::IsValidEdge */
+
+
+bool _specc::event::ResetIsActive(	/* asynchronous reset signal check */
+	bool	/* ResetActiveHi */)	/* (dummy to be overloaded) */
+{
+
+assert(false);	// pure event must not be used as asynchronous reset!
+
+return(false);	// should never be executed!
+		// (only overloaded function will!)
+
+} /* end of _specc::event::ResetIsActive */
+
+
+_specc::event_ptr *_specc::event::MappedEvents(	/* get port-mapped events */
+	void)				/* (dummy to be overloaded) */
+{
+
+return(NULL);	// this is an ordinary event
+
+} /* end of _specc::event::MappedEvents */
+
+
+	/**************************/
+	/*** _specc::auto_event ***/
+	/**************************/
+
+
+_specc::auto_event::auto_event(		/* constructor #1 */
+	sim_time	Period)
+{
+
+assert(Period > 0);	/* must be positive! */
+
+_specc::auto_event::Period	= Period;
+_specc::auto_event::TriggerTime	= CurrentTime + Period;
+_specc::auto_event::Pred	= NULL;
+_specc::auto_event::Succ	= NULL;
+
+Insert();	/* insert this into the list of self-triggering events */
+
+} /* end of _specc::auto_event::auto_event */
+
+
+_specc::auto_event::~auto_event(	/* destructor */
+	void)
+{
+
+Delete();
+
+} /* end of _specc::auto_event::~auto_event */
+
+
+void _specc::auto_event::Insert(	/* insert into the sorted list */
+	void)
+{
+auto_event	*Curr;
+
+assert(Pred == NULL);
+assert(Succ == NULL);
+
+Curr = FirstAutoEvent;
+while(Curr)
+   { if (TriggerTime <= Curr->TriggerTime)
+	{ /* insert before Curr */
+	  Pred = Curr->Pred;
+	  Succ = Curr;
+	  if (Pred)
+	     { Pred->Succ = this;
+	      } /* fi */
+	  else
+	     { assert(FirstAutoEvent == Curr);
+	       FirstAutoEvent = this;
+	      } /* esle */
+	  Succ->Pred = this;
+	  return;
+	 } /* fi */
+     Curr = Curr->Succ;
+    } /* elihw */
+
+/* append this at the end */
+
+Pred = LastAutoEvent;
+Succ = NULL;
+if (Pred)
+   { Pred->Succ = this;
+    } /* fi */
+else
+   { assert(FirstAutoEvent == NULL);
+     FirstAutoEvent = this;
+    } /* esle */
+LastAutoEvent = this;
+
+} /* end of _specc::auto_event::Insert */
+
+
+void _specc::auto_event::Delete(	/* take out of the sorted list */
+	void)
+{
+
+if (Pred)
+   { Pred->Succ = Succ;
+    } /* fi */
+else
+   { assert(FirstAutoEvent == this);
+     FirstAutoEvent = Succ;
+    } /* esle */
+
+if (Succ)
+   { Succ->Pred = Pred;
+    } /* fi */
+else
+   { assert(LastAutoEvent == this);
+     LastAutoEvent = Pred;
+    } /* esle */
+
+Pred = NULL;
+Succ = NULL;
+
+} /* end of _specc::auto_event::Delete */
+
+
+void _specc::auto_event::TriggerAndUpdate(	/* trigger, update, re-order */
+	void)
+{
+
+Notify();			/* notify this event */
+
+assert(TriggerTime == CurrentTime);
+TriggerTime += Period;		/* update the triggering time for next time */
+
+if (  (Succ)
+    &&(Succ->TriggerTime < TriggerTime))
+   { Delete();			/* re-insert if necessary */
+     Insert();
+    } /* fi */
+
+} /* end of _specc::auto_event::TriggerAndUpdate */
 
 
 	/*************************/
@@ -653,12 +990,15 @@ Next = NULL;
 	/*************************/
 
 
-_specc::event_ptr::event_ptr(	/* constructor #1 */
-	event	*Event)
+_specc::event_ptr::event_ptr(		/* constructor #1 */
+	event		*Event,
+	EDGE		Edge /* = EDGE_ANY */,
+	event_ptr	*Next /* = NULL */)
 {
 
-_specc::event_ptr::Next		= NULL;
+_specc::event_ptr::Next		= Next;
 _specc::event_ptr::Event	= Event;
+_specc::event_ptr::Edge		= Edge;
 
 } /* end of _specc::event_ptr::event_ptr */
 
@@ -695,29 +1035,36 @@ delete EventList;
 } /* end of _specc::event_list::~event_list */
 
 
-bool _specc::event_list::Triggers(	/* check whether any event matches */
-	event_ptr	*EventList2)
+_specc::event *_specc::event_list::Triggers(	/* check if any event matches */
+	event_ptr	*EventList2)	/* (returns first match, or NULL) */
 {
 event_ptr	*Event1,
 		*Event2;
-bool		Result;
+_specc::event	*Result;
+
+// note:
+//	this function is only used for the Notify1List;
+//	therefore, _this_ list does not have any 'falling'
+//	or 'rising' filters
 
 if ((!this) || (!EventList2))
-   { return(false);
+   { return(NULL);
     } /* fi */
 
 Event1 = EventList;	/* set flags up */
 while(Event1)
    { assert(Event1->Event->Triggered == false);	/* must be unused */
+     assert(Event1->Edge == EDGE_ANY);	// see note above!
      Event1->Event->Triggered = true;
      Event1 = Event1->Next;
     } /* elihw */
 
-Result = false;
+Result = NULL;
 Event2 = EventList2;	/* check for match */
 while(Event2)
-   { if (Event2->Event->Triggered)
-	{ Result = true;
+   { if (  (Event2->Event->Triggered)
+	 &&(Event2->Event->IsValidEdge(Event2->Edge)))
+	{ Result = Event2->Event;
 	  break;
 	 } /* fi */
      Event2 = Event2->Next;
@@ -835,11 +1182,11 @@ _specc::try_block::~try_block(		/* destructor */
 _specc::exception_block::exception_block(	/* constructor #1 */
 	bool		IsTrap,
 	behavior	*Behavior,
-	event		*First,
+	event_ptr	*First,
 	...)
 {
 va_list		Args;
-event		*Event;
+event_ptr	*EventPtr;
 event_ptr	**EventPtrPtr;
 
 // note: Behavior may be NULL indicating an empty exception handler
@@ -848,15 +1195,19 @@ _specc::exception_block::IsTrap		= IsTrap;
 _specc::exception_block::Behavior	= Behavior;
 _specc::exception_block::EventList	= NULL;
 
-va_start(Args, First);		/* copy event list */
-Event = First;
+// note:
+//	the caller is assumed to have properly allocated new event_ptr
+//	objects for the argument list; therefore, we just have to
+//	concatenate the arguments and store the list for the time it is used;
+//	after its use, we will delete the list
+
+va_start(Args, First);		/* traverse event list */
+EventPtr = First;
 EventPtrPtr = &EventList;
-while(Event)
-   { if (!(*EventPtrPtr = new _specc::event_ptr(Event)))
-	{ fatalError("Cannot build event list", ENOMEM);
-	 } /* fi */
-     EventPtrPtr = &(*EventPtrPtr)->Next;
-     Event = va_arg(Args, event*);
+while(EventPtr)
+   { *EventPtrPtr = EventPtr;
+     EventPtrPtr = &EventPtr->Next;
+     EventPtr = va_arg(Args, event_ptr*);
     } /* elihw */
 va_end(Args);
 
@@ -911,6 +1262,11 @@ event_ptr	*Event1,
 exception	*Exception,
 		*Result;
 
+// note:
+//	this function is only used for the Notify1List;
+//	therefore, EventList2 does not have any 'falling'
+//	or 'rising' filters
+
 if ((!this) || (!EventList2))
    { return(NULL);
     } /* fi */
@@ -918,6 +1274,7 @@ if ((!this) || (!EventList2))
 Event2 = EventList2;	/* set flags up */
 while(Event2)
    { assert(Event2->Event->Triggered == false);	/* must be unused */
+     assert(Event2->Edge == EDGE_ANY);	// see note above!
      Event2->Event->Triggered = true;
      Event2 = Event2->Next;
     } /* elihw */
@@ -927,7 +1284,8 @@ Exception = this;
 while(Exception)
    { Event1 = Exception->EventList;
      while(Event1)
-	{ if (Event1->Event->Triggered)
+	{ if (  (Event1->Event->Triggered)
+	      &&(Event1->Event->IsValidEdge(Event1->Edge)))
 	     { Result = Exception;
 	       break;
 	      } /* fi */
@@ -960,7 +1318,8 @@ Exception = this;	/* search for match */
 while(Exception)
    { Event = Exception->EventList;
      while(Event)
-	{ if (Event->Event->Notified)
+	{ if (  (Event->Event->IsNotified())
+	      &&(Event->Event->IsValidEdge(Event->Edge)))
 	     { return(Exception);
 	      } /* fi */
 	  Event = Event->Next;
@@ -970,7 +1329,7 @@ while(Exception)
 
 return(NULL);
 
-} /* end of _specc::exception_block::FirstMatch #2 */
+} /* end of _specc::exception::FirstMatch #2 */
 
 
 	/**************************/
@@ -995,12 +1354,110 @@ _specc::piped_base::~piped_base(	/* destructor */
 } /* end of _specc::piped_base::~piped_base */
 
 
-	/******************************/
-	/*** _specc::piped template ***/
-	/******************************/
+	/*****************************/
+	/*** _specc::buffered_base ***/
+	/*****************************/
 
 
-/* (all inline functions, see header file "piped.h") */
+_specc::buffered_base::buffered_base(	/* constructor #1 (for 'buffered') */
+	event_ptr	*UpdateEvents,
+	event		*ResetSignal /* = NULL */,
+	bool		ResetActiveHi /* = false */)
+{
+
+// register this with the global list of 'buffered' variables
+// (insert at the beginning of BufferedVars)
+
+buffered_base::Next = BufferedVars;
+buffered_base::Prev = NULL;
+
+if (BufferedVars)
+   { assert(BufferedVars->Prev == NULL);
+     BufferedVars->Prev = this;
+    } /* fi */
+BufferedVars = this;
+
+buffered_base::UpdateEvents	= UpdateEvents;
+buffered_base::ResetSignal	= ResetSignal;
+buffered_base::ResetActiveHi	= ResetActiveHi;
+
+} /* end of _specc::buffered_base::buffered_base #1 */
+
+
+_specc::buffered_base::buffered_base(	/* constructor #2 (for 'signal') */
+	event		*SignalEvent)
+{
+
+// register this with the global list of 'signal' variables
+// (insert at the beginning of SignalVars)
+
+buffered_base::Next = SignalVars;
+buffered_base::Prev = NULL;
+
+if (SignalVars)
+   { assert(SignalVars->Prev == NULL);
+     SignalVars->Prev = this;
+    } /* fi */
+SignalVars = this;
+
+buffered_base::UpdateEvents	= new _specc::event_ptr(SignalEvent);
+buffered_base::ResetSignal	= NULL;		// no reset for signals
+buffered_base::ResetActiveHi	= false;	// no reset for signals
+
+} /* end of _specc::buffered_base::buffered_base #2 */
+
+
+_specc::buffered_base::~buffered_base(	/* destructor */
+	void)
+{
+
+// un-register this with the proper global list of 'buffered'
+// or 'signal' variables
+
+if (Prev)
+   { Prev->Next = Next;
+    } /* fi */
+else
+   { assert(  ((BufferedVars == this)&&(SignalVars != this))
+	    ||((BufferedVars != this)&&(SignalVars == this)));
+     if (BufferedVars == this)
+	{ BufferedVars = Next;
+	 } /* fi */
+     else
+	{ SignalVars = Next;
+	 } /* esle */
+    } /* esle */
+if (Next)
+   { Next->Prev = Prev;
+    } /* fi */
+
+delete UpdateEvents;
+
+} /* end of _specc::buffered_base::~buffered_base */
+
+
+	/**********************/
+	/*** piped template ***/
+	/**********************/
+
+
+/* (see header file "piped.h") */
+
+
+	/*************************/
+	/*** buffered template ***/
+	/*************************/
+
+
+/* (see header file "signals.h") */
+
+
+	/***********************/
+	/*** signal template ***/
+	/***********************/
+
+
+/* (see header file "signals.h") */
 
 
 /************************************************************************/
